/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model;

import java.io.Serializable;

/**
 * Categoria, almacena la configuracin para mostrar un conjunto de informacin relacionada.
 */
public class Categoria implements Serializable {

	/** La constante serialVersionUID. */
	private static final long serialVersionUID = 3248509399689535242L;

	/** Consulta para obtener la uri. */
	private Consulta consultaUri;

	/** Fichero de configuracin. */
	private String ficheroConf;

	/** The imagen. */
	private String imagen;

	/** The nombre. */
	private String nombre;

	/** The uri. */
	private String uri;

	/**
	 * Obtiene la consulta para obtener la uri.
	 *
	 * @return consulta para obtener la uri
	 */
	public Consulta getConsultaUri() {
		return this.consultaUri;
	}

	/**
	 * Obtiene el nombre del fichero de configuracin de la categora.
	 *
	 * @return fichero de configuracin
	 */
	public String getFicheroConf() {
		return this.ficheroConf;
	}

	/**
	 * Obtiene la imagen.
	 *
	 * @return la imagen.
	 */
	public String getImagen() {
		return this.imagen;
	}

	/**
	 * Obtiene el nombre de la categora.
	 *
	 * @return el nombre
	 */
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Obtiene la uri.
	 *
	 * @return la uri
	 */
	public String getUri() {
		return this.uri;
	}

	/**
	 * Fija la consulta para obtener la uri.
	 *
	 * @param consultaUri
	 *            consulta para obtener la uri
	 */
	public void setConsultaUri(Consulta consultaUri) {
		this.consultaUri = consultaUri;
	}

	/**
	 * Fija el fichero de configuracin.
	 *
	 * @param ficheroConf
	 *            el nuevo fichero de configuracin
	 */
	public void setFicheroConf(String ficheroConf) {
		this.ficheroConf = ficheroConf;
	}

	/**
	 * Fija la imagen.
	 *
	 * @param imagen
	 *            la nueva imagen
	 */
	public void setImagen(String imagen) {
		this.imagen = imagen;
	}

	/**
	 * Fija el nombre de la categora.
	 *
	 * @param nombre
	 *            el nuevo nombre de la categora
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	/**
	 * Fija la uri.
	 *
	 * @param uri
	 *            la nueva uri
	 */
	public void setUri(String uri) {
		this.uri = uri;
	}

}
