/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * ListaGeoPosiciones.
 * Almacena la lista de posiciones de una ciudad
 */
public class ListaGeoPosiciones implements Serializable {

	/** La constante serialVersionUID. */
	private static final long serialVersionUID = -8669342620365471154L;

	/** El centro de la ciudad, indica donde se centrar el mapa mostrado con todos los puntos */
	private GeoPosicion centro;

	/** El fichero HDT de la ciudad. */
	private FicheroHdt fichero;

	/** La lista de puntos. */
	private List<GeoPosicionEtiqueta> listaPuntos = new ArrayList<GeoPosicionEtiqueta>();

	/** El nombre. */
	private String nombre;

	/**
	 * Aade una posicin a la lista
	 *
	 * @param posicion
	 *            la posicion
	 * @param etiqueta
	 *            la etiqueta de la poscin aadida
	 */
	public void add(GeoPosicion posicion, String etiqueta) {
		this.listaPuntos.add(new GeoPosicionEtiqueta(posicion, etiqueta));
	}

	/**
	 * Aade el una posicin seleccionable a la lista
	 *
	 * @param posicion
	 *            la posicin
	 * @param etiqueta
	 *            el nombre de la posicin
	 * @param uri
	 *            la uri del recurso correspondiente a la posicin
	 * @param ciudad
	 *            la ciudad a la que pertenece la posicin, o fichero HDT donde realizar las consultas para obtener la informacin de la posicin
	 */
	public void add(GeoPosicion posicion, String etiqueta, String uri, FicheroHdt ciudad) {
		GeoPosicionEtiqueta geoPosicionEtiqueta = new GeoPosicionEtiqueta(posicion, etiqueta);
		geoPosicionEtiqueta.setUri(uri);
		geoPosicionEtiqueta.setCiudad(ciudad);
		this.listaPuntos.add(geoPosicionEtiqueta);
	}

	/**
	 * Aade una posicin.
	 *
	 * @param posicionEtiquetada
	 *            objeto de posicin, con la etiqueta
	 */
	public void add(GeoPosicionEtiqueta posicionEtiquetada) {
		this.listaPuntos.add(posicionEtiquetada);
	}

	/**
	 * Limpia la lista de posiciones.
	 */
	public void clear() {
		this.listaPuntos.clear();
	}

	/**
	 * Obtiene el centro.
	 *
	 * @return el centro
	 */
	public GeoPosicion getCentro() {
		if (this.centro == null && this.listaPuntos.size() > 0) {
			return this.listaPuntos.get(0).getPosicion();
		}
		return this.centro;
	}

	/**
	 * Obtiene el fichero HDT
	 *
	 * @return the fichero
	 */
	public FicheroHdt getFichero() {
		return this.fichero;
	}

	/**
	 * Obtiene la lista de puntos
	 *
	 * @return the listaPuntos
	 */
	public List<GeoPosicionEtiqueta> getListaPuntos() {
		return this.listaPuntos;
	}

	/**
	 * Gets the nombre.
	 *
	 * @return the nombre
	 */
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Num posiciones.
	 *
	 * @return the int
	 */
	public int numPosiciones() {
		return this.listaPuntos.size();
	}

	/**
	 * Sets the centro.
	 *
	 * @param centro
	 *            the new centro
	 */
	public void setCentro(GeoPosicion centro) {
		this.centro = centro;
	}

	/**
	 * Sets the fichero.
	 *
	 * @param fichero
	 *            the new fichero
	 */
	public void setFichero(FicheroHdt fichero) {
		this.fichero = fichero;
	}

	/**
	 * Sets the lista puntos.
	 *
	 * @param listaPuntos
	 *            the listaPuntos to set
	 */
	public void setListaPuntos(List<GeoPosicionEtiqueta> listaPuntos) {
		this.listaPuntos = listaPuntos;
	}

	/**
	 * Sets the nombre.
	 *
	 * @param nombre
	 *            the nombre to set
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
}
