/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.hdtourist.R;
import org.hdtourist.model.Ciudad;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.xml.FicherosConfigParser;
import org.hdtourist.utils.Constantes;
import org.hdtourist.utils.FileUtils;
import org.hdtourist.utils.HdtException;
import org.hdtourist.utils.HdtFilter;
import org.hdtourist.view.utils.HdTIdioma;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.util.Log;

/**
 * El servicio que proporciona utilidades relacionadas con los ficheros.
 */
public class FicherosService {

	/** La Constante BUFFER_SIZE. */
	public static final int BUFFER_SIZE = 1024 * 200;

	/** La instancia. */
	private static FicherosService INSTANCE = null;

	/** La Constante LOG_TAG. */
	private static final String LOG_TAG = "FicherosService";

	/**
	 * Instancia un nuevo servicio de ficheros
	 */
	private FicherosService() {

	}

	/**
	 * Combina varios ficheros descriptores de hdts descargados.
	 * Util a la hora de mover el directorio de archivos de configuracin a otro directorio que ya contena ficheros de configuracin previos
	 *
	 * @param descriptorFinal
	 *            el descriptor final
	 * @param descriptorAnterior
	 *            el descriptor anterior
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void combinarDescriptorHdtsDescargados(File descriptorFinal, File descriptorAnterior) throws HdtException {

		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(descriptorFinal);
			Node ficheros = doc.getFirstChild();

			List<Element> elementosPorAadir = new ArrayList<Element>();

			DocumentBuilderFactory docFactoryAnt = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilderAnt = docFactoryAnt.newDocumentBuilder();
			Document docAnt = docBuilderAnt.parse(descriptorAnterior);

			NodeList nodesAnt = docAnt.getElementsByTagName("fichero");
			NodeList nodesFinal = doc.getElementsByTagName("fichero");
			for (int i = 0; i < nodesAnt.getLength(); i++) {
				Element ficheroAnt = (Element) nodesAnt.item(i);
				String nombreCiudadAnt = ((Element) ficheroAnt.getElementsByTagName("nombreCiudad").item(0)).getTextContent();
				boolean esta = false;
				for (int j = 0; j < nodesFinal.getLength(); j++) {
					Element ficheroF = (Element) nodesFinal.item(j);
					String nombreCiudadF = ((Element) ficheroF.getElementsByTagName("nombreCiudad").item(0)).getTextContent();
					if (nombreCiudadF.equals(nombreCiudadAnt)) {
						esta = true;
					}
				}
				if (!esta) {
					elementosPorAadir.add(ficheroAnt);
				}
			}

			for (Element fichero : elementosPorAadir) {
				ficheros.appendChild(fichero);
			}
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(descriptorFinal.getAbsolutePath());
			transformer.transform(source, result);
		} catch (Exception e) {

		}
	}

	/**
	 * Devuelve true si se tienen los permisos de escritura necesarios.
	 *
	 * @return true, if ha tenido exito
	 */
	public static boolean comprobarMemoria() {
		String stadoSD = Environment.getExternalStorageState();

		boolean sdEscritura = false;
		boolean sdLectura = false;
		if (stadoSD.equals(Environment.MEDIA_MOUNTED)) {
			sdEscritura = true;
			sdLectura = true;
		} else if (stadoSD.equals(Environment.MEDIA_MOUNTED_READ_ONLY)) {
			sdEscritura = false;
			sdLectura = true;
		} else {
			sdEscritura = false;
			sdLectura = false;
		}

		return sdEscritura && sdLectura;
	}

	/**
	 * Copia un directorio del directorio destino al directorio final.
	 *
	 * @param src
	 *            directorio final
	 * @param dest
	 *            directio destion
	 * @throws IOException
	 *             Excepcin I/O
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void copyFolder(File src, File dest) throws IOException, HdtException {

		if (src.isDirectory()) {

			// if directory not exists, create it
			if (!dest.exists()) {
				dest.mkdir();
			}

			// list all the directory contents
			String files[] = src.list();

			for (String file : files) {
				// construct the src and dest file structure
				File srcFile = new File(src, file);
				File destFile = new File(dest, file);
				// recursive copy
				FicherosService.copyFolder(srcFile, destFile);
			}

		} else {
			InputStream in = null;
			OutputStream out = null;
			try {
				if (dest.exists() && src.getName().equals(FicherosService.obtenDescriptorHdtsDescargados().getName())) {
					FicherosService.combinarDescriptorHdtsDescargados(src, dest);
					return;
				}
				in = new FileInputStream(src);
				out = new FileOutputStream(dest);

				byte[] buffer = new byte[1024];

				int length;
				// copy the file content in bytes
				while ((length = in.read(buffer)) > 0) {
					out.write(buffer, 0, length);
				}
			} finally {
				if (in != null) {
					in.close();
				}
				if (out != null) {
					out.close();
				}
			}
		}
	}

	/**
	 * Copia un recurso RAW interno del programa a la tarjeta de memoria
	 *
	 * @param id
	 *            id del recurso a copiar
	 * @param path
	 *            la ruta donde se copiar el recurso
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void copyRAWtoSDCard(int id, String path) throws HdtException {
		try {
			InputStream in = HdTouristService.getInstance().getContexto().getResources().openRawResource(id);
			FileOutputStream out = new FileOutputStream(path);
			byte[] buff = new byte[1024];
			int read = 0;
			try {
				while ((read = in.read(buff)) > 0) {
					out.write(buff, 0, read);
				}
			} finally {
				in.close();
				out.close();
			}
		} catch (IOException e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_copiando_archivos_configuracion));
		}
	}

	/**
	 * Crea el directorio si no existe, controlando los permisos de escrtura.
	 *
	 * @param dir
	 *            Directorio a crear si no existe
	 * @throws HdtException
	 *             Si no hay permisos de escritura se lanza una excepcin
	 */
	public static void creaDirSiNoExiste(File dir) throws HdtException {
		if (!dir.exists() && FicherosService.comprobarMemoria()) {
			dir.mkdirs();
		} else if (!dir.exists()) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_memoria));
		}
	}

	/**
	 * Eliminar fichero descargado.
	 *
	 * @param ficheroHdt
	 *            el fichero hdt
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void eliminarFicheroDescargado(FicheroHdt ficheroHdt) throws HdtException {
		try {
			File ficheroSalida = FicherosService.obtenDescriptorHdtsDescargados();
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(ficheroSalida);

			Node ficheros = doc.getFirstChild();

			NodeList nodes = doc.getElementsByTagName("fichero");
			for (int i = 0; i < nodes.getLength(); i++) {
				Element fichero = (Element) nodes.item(i);
				Element nombreCiudad = (Element) fichero.getElementsByTagName("nombreCiudad").item(0);
				String nombreCiudadText = nombreCiudad.getTextContent();
				if (ficheroHdt.getNombre().equals(nombreCiudadText)) {
					ficheros.removeChild(fichero);
				}
			}

			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(ficheroSalida.getAbsolutePath());
			transformer.transform(source, result);
		} catch (Exception e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_eliminar_hdt));
		}
	}

	/**
	 * Consulta si el fichero HDT descargado ya estaba guardado previamente.
	 *
	 * @param ficheroHdt
	 *            el fichero hdt
	 * @return true, si el fichero HDT ya estaba guardado
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static boolean ficheroDescargadoYaGuardado(FicheroHdt ficheroHdt) throws HdtException {
		try {
			File ficheroSalida = FicherosService.obtenDescriptorHdtsDescargados();
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(ficheroSalida);

			NodeList nodes = doc.getElementsByTagName("fichero");
			for (int i = 0; i < nodes.getLength(); i++) {
				Element fichero = (Element) nodes.item(i);
				Element nombreCiudad = (Element) fichero.getElementsByTagName("nombreCiudad").item(0);
				String nombreCiudadText = nombreCiudad.getTextContent();
				if (ficheroHdt.getNombre().equals(nombreCiudadText)) {
					return true;
				}
			}
			return false;
		} catch (Exception e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_eliminar_hdt));
		}
	}

	/**
	 * Obtiene la imagen de la categoria.
	 *
	 * @param ficheroImagen
	 *            nombre del fichero de la imagen
	 * @return the imagen categoria
	 */
	public static Bitmap getImagenCategoria(String ficheroImagen) {
		// TODO, comprobar que pasa si no existe el fichero, obtenerlo de la memoria
		int id = HdTouristService.getInstance().getContexto().getResources()
				.getIdentifier(ficheroImagen, "drawable", HdTouristService.getInstance().getContexto().getPackageName());
		return BitmapFactory.decodeResource(HdTouristService.getInstance().getContexto().getResources(), id);
	}

	/**
	 * Obtiene la isntancia del servicio de ficheros.
	 *
	 * @return la instancia del servicio
	 */
	public static FicherosService getInstance() {
		FicherosService.createInstance();
		return FicherosService.INSTANCE;
	}

	/**
	 * Obtiene el tamao del fichero pasado a cadena, indicando el tamao correspondiente.
	 *
	 * @param size
	 *            el tamao
	 * @return la cadena que indica el tamao
	 */
	public static String getStringSize(BigDecimal size) {
		if (size == null || size.floatValue() < 0) {
			return HdTIdioma.getString(R.string.tam_no_conocido);
		}

		float s = size.floatValue();
		BigDecimal m = new BigDecimal("1024");
		if (s == 0) {
			return HdTIdioma.getString(R.string.tam_no_existe);
		}
		if (s < 1024) {
			return size.toString() + " " + HdTIdioma.getString(R.string.tam_b);
		} else if (s < 1024 * 1024) {
			return size.divide(m, 2, RoundingMode.HALF_UP).toString() + " " + HdTIdioma.getString(R.string.tam_kb);
		} else if (s < 1024 * 1024 * 1024) {
			return size.divide(m.multiply(m), 2, RoundingMode.HALF_UP).toString() + " " + HdTIdioma.getString(R.string.tam_mb);
		}

		return size.divide(m.multiply(m).multiply(m), 2, RoundingMode.HALF_UP).toString() + " " + HdTIdioma.getString(R.string.tam_gb);

	}

	/**
	 * Guarda la informacin del fichero descargado.
	 *
	 * @param ficheroHdt
	 *            el fichero HDT descargado
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void guardaFicheroDescargado(FicheroHdt ficheroHdt) throws HdtException {
		try {
			if (FicherosService.ficheroDescargadoYaGuardado(ficheroHdt)) {
				return;
			}
			File ficheroSalida = FicherosService.obtenDescriptorHdtsDescargados();
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(ficheroSalida);
			Node ficheros = doc.getFirstChild();
			Element fichero = doc.createElement("fichero");

			Element nombre = doc.createElement("nombre");
			nombre.appendChild(doc.createTextNode(ficheroHdt.getFichero().getName()));
			fichero.appendChild(nombre);

			Element uri = doc.createElement("uri");
			uri.appendChild(doc.createTextNode(ficheroHdt.getUri()));
			fichero.appendChild(uri);

			Element size = doc.createElement("size");
			size.appendChild(doc.createTextNode(ficheroHdt.getTamano().intValue()+""));
			fichero.appendChild(size);

			Element size2 = doc.createElement("sizeMaps");
			size2.appendChild(doc.createTextNode(ficheroHdt.getTamanoMaps().intValue()+""));
			fichero.appendChild(size2);

			Element nombreCiudad = doc.createElement("nombreCiudad");
			nombreCiudad.appendChild(doc.createTextNode(ficheroHdt.getNombre()));
			fichero.appendChild(nombreCiudad);

			ficheros.appendChild(fichero);

			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(ficheroSalida.getAbsolutePath());
			transformer.transform(source, result);

		} catch (Exception e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_guardando_hdt));
		}
	}

	/**
	 * Mueve los archivos de configuracion.
	 *
	 * @param antiguoPath
	 *            ruta donde estn actualmente los ficheros
	 * @param nuevoPath
	 *            nueva ruta hacia donde se movern los ficheros
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static void moverArchivosConfiguracion(String antiguoPath, String nuevoPath) throws HdtException {
		if (antiguoPath.equals(nuevoPath)) {
			return;
		}
		File antiguaRuta = new File(antiguoPath);
		File nuevaRuta = new File(nuevoPath);
		try {
			FicherosService.copyFolder(antiguaRuta, nuevaRuta);
			FileUtils.forceDelete(antiguoPath);
		} catch (IOException e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_copiando_archivos));
		}

	}

	/**
	 * Obtiene el archivo de informacin de los fihceros hdts descargados.
	 *
	 * @return el archivo
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static File obtenDescriptorHdtsDescargados() throws HdtException {
		File ruta_sd = new File(FicherosService.getInstance().obtenDirectorioConfiguracion().getAbsolutePath() + File.separator
				+ Constantes.HDT_CIUDADES_XML);
		;
		if (!ruta_sd.exists()) {
			try {

				DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
				Document doc = docBuilder.newDocument();
				Element rootElement = doc.createElement("ficheros");
				doc.appendChild(rootElement);

				TransformerFactory transformerFactory = TransformerFactory.newInstance();
				Transformer transformer = transformerFactory.newTransformer();
				DOMSource source = new DOMSource(doc);
				StreamResult result = new StreamResult(ruta_sd);
				transformer.transform(source, result);
			} catch (Exception e) {
				Log.e(FicherosService.LOG_TAG, "Ocurri un error obteniendo el descriptor de Hdts descargados", e);
			}
		}
		return ruta_sd;
	}

	/**
	 * Obtiene el directorio donde sen encuentran almacenados los ficheros HDT en el dispositivo.
	 *
	 * @return el directorio
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static File obtenDirectorioHdts() throws HdtException {
		File ruta_sd = new File(PreferencesService.getDirectorio() + File.separator + Constantes.HDT_CARPETA);
		FicherosService.creaDirSiNoExiste(ruta_sd);
		return ruta_sd;
	}

	/**
	 * Obtiene el directorio donde sen encuentran almacenados los ficheros HDT en el dispositivo.
	 *
	 * @return la cadena que representa el directorio por defecto
	 */
	public static String obtenDirectorioPorDefecto() {
		return HdTouristService.getInstance().getContexto().getExternalFilesDir(null).getPath();
	}

	/**
	 * Obtiene la lista de todos los ficheros HDT que se encuentran en el dispositivo.
	 *
	 * @return la lista
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static ArrayList<FicheroHdt> obtenFicherosHdtDescargados() throws HdtException {
		ArrayList<FicheroHdt> result = new ArrayList<FicheroHdt>();

		if (!FicherosService.comprobarMemoria()) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_memoria));
		}

		File ruta_sd = FicherosService.obtenDirectorioHdts();
		String[] s = ruta_sd.list(new HdtFilter());
		if (s == null) {
			return result;
		}
		FicherosConfigParser configParser = FicherosConfigParser.getInstance();
		configParser.setFile(FicherosService.obtenDescriptorHdtsDescargados());
		List<Ciudad> ficheros = configParser.obtenerFicheros();
		Iterator<Ciudad> i = ficheros.iterator();
		while (i.hasNext()) {
			Ciudad ficheroConfig = i.next();
			FicheroHdt f = new FicheroHdt();
			f.setRutaFichero(ruta_sd + File.separator + ficheroConfig.getNombre());
			f.setNombre(ficheroConfig.getNombreCiudad());
			f.setUri(ficheroConfig.getUri());
			f.setTamano(ficheroConfig.getSize());
			f.setTamanoMaps(ficheroConfig.getSizeMaps());
			if (!f.getFichero().exists()) {
				continue;
			}
			result.add(f);
		}
		return result;
	}

	/**
	 * Crea la instancia.
	 */
	private synchronized static void createInstance() {

		if (FicherosService.INSTANCE == null) {
			FicherosService.INSTANCE = new FicherosService();
		}
	}

	/**
	 * Obtiene el directorio donde se encuentran almacenados los archivos de configuracin.
	 *
	 * @return el directorio
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public File obtenDirectorioConfiguracion() throws HdtException {
		File ruta_sd = new File(PreferencesService.getDirectorio() + File.separator + Constantes.CONF_CARPETA);
		FicherosService.creaDirSiNoExiste(ruta_sd);
		return ruta_sd;
	}

	/**
	 * Obtiene el directorio donde se encuentran almacenada la cach de las imagenes.
	 *
	 * @return el directorio
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public File obtenDirectorioImagen() throws HdtException {
		File ruta_sd = new File(PreferencesService.getDirectorio() + File.separator + Constantes.CONF_IMAGENES);
		FicherosService.creaDirSiNoExiste(ruta_sd);
		return ruta_sd;
	}

	/**
	 * Obtiene el directorio donde se encuentran almacenados los mapas en el dispositivo.
	 *
	 * @return el directorio
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public File obtenDirectorioMapas() throws HdtException {
		File ruta_sd = new File(PreferencesService.getDirectorio() + File.separator + Constantes.MAPAS_CARPETA);
		FicherosService.creaDirSiNoExiste(ruta_sd);
		return ruta_sd;
	}

}
