/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;

/**
 * Servicio que proporciona funciones relacionadas con los ficheros HDT.
 */
public class HdTouristService {

	/** El contexto. */
	private static Context contexto;

	/** The instance. */
	private static HdTouristService INSTANCE = null;

	/** The lista procesos. */
	private List<Runnable> listaProcesos = new ArrayList<Runnable>();

	/**
	 * Instantiates a new hd tourist service.
	 */
	private HdTouristService() {

	}

	/**
	 * Gets the single instance of HdTouristService.
	 *
	 * @return single instance of HdTouristService
	 */
	public static HdTouristService getInstance() {
		HdTouristService.createInstance(null);
		return HdTouristService.INSTANCE;
	}

	/**
	 * Gets the single instance of HdTouristService.
	 *
	 * @param activity
	 *            the activity
	 * @return single instance of HdTouristService
	 */
	public static HdTouristService getInstance(Activity activity) {
		HdTouristService.createInstance(activity.getApplicationContext());
		return HdTouristService.INSTANCE;
	}

	/**
	 * Crea el instance.
	 *
	 * @param contexto
	 *            the contexto
	 */
	private synchronized static void createInstance(Context contexto) {
		if (contexto != null) {
			HdTouristService.contexto = contexto;
		}
		if (HdTouristService.INSTANCE == null) {
			HdTouristService.INSTANCE = new HdTouristService();
		}
	}

	/**
	 * Aade el proceso.
	 *
	 * @param proceso
	 *            the proceso
	 */
	public void addProceso(Runnable proceso) {
		this.listaProcesos.add(proceso);
	}

	/**
	 * Gets the contexto.
	 *
	 * @return the contexto
	 */
	public Context getContexto() {
		return HdTouristService.contexto;
	}

	/**
	 * Checks if is network available.
	 *
	 * @return true, if is network available
	 */
	public boolean isNetworkAvailable() {
		ConnectivityManager connectMgr = (ConnectivityManager) HdTouristService.contexto.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectMgr != null) {
			NetworkInfo[] netInfo = connectMgr.getAllNetworkInfo();
			if (netInfo != null) {
				for (NetworkInfo net : netInfo) {
					if (net.getState() == NetworkInfo.State.CONNECTED) {
						return true;
					}
				}
			}
		} else {
			Log.d("NETWORK", "No network available");
		}
		return false;
	}

	/**
	 * Elimina el proceso.
	 *
	 * @param proceso
	 *            the proceso
	 */
	public void removeProceso(Runnable proceso) {
		this.listaProcesos.remove(proceso);
	}
}
