/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services.xml;

import java.util.ArrayList;
import java.util.List;

import org.hdtourist.model.Categoria;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * El objeto CategoriasConfigHandler.
 */
public class CategoriasConfigHandler extends DefaultHandler {

	/** La categora actual. */
	private Categoria categoriaActual;

	/** La lista de categoras. */
	private List<Categoria> categorias;

	/** Texto cache de almacenamiento. */
	private StringBuilder sbTexto;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {

		super.characters(ch, start, length);

		if (this.categoriaActual != null) {
			this.sbTexto.append(ch, start, length);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {

		super.endElement(uri, localName, name);

		if (this.categoriaActual != null) {

			if (localName.equals("nombre")) {
				this.categoriaActual.setNombre(this.sbTexto.toString().trim());
			} else if (localName.equals("fichero")) {
				this.categoriaActual.setFicheroConf(this.sbTexto.toString().trim());
			} else if (localName.equals("imagen")) {
				this.categoriaActual.setImagen(this.sbTexto.toString().trim());
			} else if (localName.equals("categoria")) {
				this.categorias.add(this.categoriaActual);
			}

			this.sbTexto.setLength(0);
		}
	}

	/**
	 * La lista de categorias.
	 *
	 * @return las categorias
	 */
	public List<Categoria> getCategorias() {
		return this.categorias;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startDocument()
	 */
	@Override
	public void startDocument() throws SAXException {

		super.startDocument();

		this.categorias = new ArrayList<Categoria>();
		this.sbTexto = new StringBuilder();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {

		super.startElement(uri, localName, name, attributes);

		if (localName.equals("categoria")) {
			this.categoriaActual = new Categoria();
		}
	}
}
