/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.hdtourist.R;
import org.hdtourist.model.Categoria;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;

/**
 * El objeto CategoriasConfigParser.
 */
public class CategoriasConfigParser {

	/** La instance. */
	private static CategoriasConfigParser INSTANCE = null;

	/**
	 * Instancia una nueva categoria config parser.
	 */
	private CategoriasConfigParser() {
	}

	/**
	 * Obtiene la nica instnacia CategoriasConfigParser.
	 *
	 * @return nica instancia de CategoriasConfigParser
	 */
	public static CategoriasConfigParser getInstance() {
		CategoriasConfigParser.createInstance();
		return CategoriasConfigParser.INSTANCE;
	}

	/**
	 * Crea la instancia.
	 */
	private synchronized static void createInstance() {
		if (CategoriasConfigParser.INSTANCE == null) {
			CategoriasConfigParser.INSTANCE = new CategoriasConfigParser();
		}
	}

	/**
	 * Obtener categorias.
	 *
	 * @return la lista de categoras
	 */
	public List<Categoria> obtenerCategorias() {
		SAXParserFactory factory = SAXParserFactory.newInstance();

		try {
			SAXParser parser = factory.newSAXParser();
			CategoriasConfigHandler handler = new CategoriasConfigHandler();
			parser.parse(this.getInputStream(), handler);
			return handler.getCategorias();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Obtiene el inputStream del fichero xml
	 *
	 * @return the input stream
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private InputStream getInputStream() throws IOException {
		String fichero = "conf_ciudad";
		try {
			File fichConf = new File(FicherosService.getInstance().obtenDirectorioConfiguracion().getAbsolutePath() + File.separator + fichero
					+ ".xml");
			if (fichConf.exists()) {
				return new FileInputStream(fichConf);
			} else {
				InputStream fraw = HdTouristService.getInstance().getContexto().getResources().openRawResource(R.raw.conf_ciudad);
				return fraw;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

	}
}
