/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.hdtourist.model.Ciudad;

/**
 * El objeto FicherosConfigParser.
 */
public class FicherosConfigParser {

	/** The instance. */
	private static FicherosConfigParser INSTANCE = null;

	/** The fichero xml. */
	private File ficheroXml;

	/**
	 * Instancia una nuevo ficheros config parser.
	 */
	private FicherosConfigParser() {
	}

	/**
	 * Gets the single instance of FicherosConfigParser.
	 *
	 * @return single instance of FicherosConfigParser
	 */
	public static FicherosConfigParser getInstance() {
		FicherosConfigParser.createInstance();
		return FicherosConfigParser.INSTANCE;
	}

	/**
	 * Crea la instanca.
	 */
	private synchronized static void createInstance() {
		if (FicherosConfigParser.INSTANCE == null) {
			FicherosConfigParser.INSTANCE = new FicherosConfigParser();
		}
	}

	/**
	 * Obtener ficheros.
	 *
	 * @return la lista de ciudades
	 */
	public List<Ciudad> obtenerFicheros() {
		SAXParserFactory factory = SAXParserFactory.newInstance();

		try {
			SAXParser parser = factory.newSAXParser();
			FicherosConfigHandler handler = new FicherosConfigHandler();
			parser.parse(this.getInputStream(), handler);
			return handler.getFicheros();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Sets the file.
	 *
	 * @param ficheroXml
	 *            the new file
	 */
	public void setFile(File ficheroXml) {
		this.ficheroXml = ficheroXml;
	}

	/**
	 * Obtiene el input stream.
	 *
	 * @return the input stream
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private InputStream getInputStream() throws IOException {
		InputStream fraw = new FileInputStream(this.ficheroXml);
		return fraw;
	}
}
