/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services.xml;

import java.util.ArrayList;
import java.util.List;

import org.hdtourist.model.Categoria;
import org.hdtourist.model.Consulta;
import org.hdtourist.model.ConsultaTuberia;
import org.hdtourist.model.Triple;
import org.hdtourist.presenter.InformacionListPresenter;
import org.hdtourist.view.inf.InformacionDato;
import org.hdtourist.view.inf.InformacionDatoEnumerado;
import org.hdtourist.view.inf.InformacionEnlace;
import org.hdtourist.view.inf.InformacionGrupo;
import org.hdtourist.view.inf.InformacionImagen;
import org.hdtourist.view.inf.InformacionLista;
import org.hdtourist.view.inf.InformacionListaVinculos;
import org.hdtourist.view.inf.InformacionMapa;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * La clase InformacionConfigHandler.
 */
public class InformacionConfigHandler extends DefaultHandler {

	/** El objeto categoria accesible. */
	private Categoria categoriaAccesible;

	/** The consulta. */
	private Consulta consulta;

	/** The consulta tuberia. */
	private ConsultaTuberia consultaTuberia;

	/** El dato. */
	private InformacionDato dato;

	/** Los datos. */
	private List<InformacionDato> datos;

	/** El grupo informacion. */
	private InformacionGrupo grupoInformacion;

	/** Los grupos informacion. */
	private List<InformacionGrupo> gruposInformacion;

	/** informacion config. */
	private InformacionListPresenter informacionConfig;

	/** The sb texto. */
	private StringBuilder sbTexto;

	/** The triple. */
	private Triple triple;

	/** The triples. */
	private List<Triple> triples;

	/** The triples tuberia. */
	private List<Triple> triplesTuberia;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {

		super.characters(ch, start, length);

		if (this.informacionConfig != null) {
			this.sbTexto.append(ch, start, length);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {

		super.endElement(uri, localName, name);

		if (this.informacionConfig != null) {

			if (localName.equals("nombre")) {
				this.informacionConfig.setNombre(this.sbTexto.toString().trim());
			} else if (localName.equals("categoria")) {
				this.informacionConfig.setListaInformacion(this.gruposInformacion);
				this.gruposInformacion = null;
			}

			if (this.triple != null) {
				if (localName.equals("sujeto")) {
					this.triple.setSujeto(this.sbTexto.toString().trim());
				} else if (localName.equals("predicado")) {
					this.triple.setPredicado(this.sbTexto.toString().trim());
				} else if (localName.equals("objeto")) {
					this.triple.setObjeto(this.sbTexto.toString().trim());
				} else if (localName.equals("consulta")) {
					if (this.consultaTuberia != null) {
						this.triplesTuberia.add(this.triple);
					} else {
						this.triples.add(this.triple);
					}
					this.triple = null;
				}
			}

			if (this.triples != null && localName.equals("consultas")) {
				this.consulta.setTriples(this.triples);
				this.triples = null;

				this.dato.setConsulta(this.consulta);
				this.consulta = null;
			}

			if (this.triples != null && localName.equals("uri")) {
				this.consulta.setTriples(this.triples);
				this.triples = null;
			}

			if (this.dato != null) {
				if (localName.equals("etiqueta")) {
					this.dato.setEtiqueta(this.sbTexto.toString().trim());
				}
				if (this.categoriaAccesible != null) {
					if (localName.equals("accesible")) {
						this.categoriaAccesible.setConsultaUri(this.consulta);
						this.dato.setCategoria(this.categoriaAccesible);
						this.categoriaAccesible = null;
						this.consulta = null;
					} else if (localName.equals("ficheroconfig")) {
						this.categoriaAccesible.setFicheroConf(this.sbTexto.toString().trim());
					}
				}
			}

			if (this.datos != null) {
				if (localName.equals("dato")) {
					this.datos.add(this.dato);
					this.dato = null;
				}
			}

			if (this.grupoInformacion != null) {
				if (localName.equals("nombreAgrupacion")) {
					this.grupoInformacion.setEtiqueta(this.sbTexto.toString().trim());
				} else if (localName.equals("listaDatos")) {
					this.grupoInformacion.setListaDatos(this.datos);
					this.datos = null;
				} else if (localName.equals("agrupacionInformacion")) {
					this.gruposInformacion.add(this.grupoInformacion);
					this.grupoInformacion = null;
				}

			}

			if (this.consultaTuberia != null) {
				if (localName.equals("tuberia")) {
					this.consultaTuberia.setTriples(this.triplesTuberia);
					this.consulta.getConsultasTuberia().add(this.consultaTuberia);
					this.consultaTuberia = null;
					this.triplesTuberia = null;
				}
			}
		}

		this.sbTexto.setLength(0);
	}

	/**
	 * Gets the informacion config.
	 *
	 * @return the informacion config
	 */
	public InformacionListPresenter getInformacionConfig() {
		return this.informacionConfig;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startDocument()
	 */
	@Override
	public void startDocument() throws SAXException {

		super.startDocument();

		this.informacionConfig = new InformacionListPresenter();
		this.sbTexto = new StringBuilder();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {

		super.startElement(uri, localName, name, attributes);

		if (localName.equals("categoria")) {
			this.gruposInformacion = new ArrayList<InformacionGrupo>();
			this.informacionConfig.setId(attributes.getValue("id"));
		} else if (localName.equals("agrupacionInformacion")) {
			this.grupoInformacion = new InformacionGrupo();
			if ("otros".equalsIgnoreCase(attributes.getValue("type"))) {
				this.grupoInformacion.setOtros(true);
			}
		} else if (localName.equals("listaDatos")) {
			this.datos = new ArrayList<InformacionDato>();
		} else if (localName.equals("dato")) {
			this.dato = new InformacionDato();
			if ("listaVinculos".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionListaVinculos();
			} else if ("enum".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionDatoEnumerado();
			} else if ("mapa".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionMapa();
			} else if ("lista".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionLista();
			} else if ("imagen".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionImagen();
			} else if ("enlace".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato = new InformacionEnlace();
			}
			if ("true".equalsIgnoreCase(attributes.getValue("onlyText"))) {
				this.dato.setOnlyText(true);
			}
			if ("imagen_text".equalsIgnoreCase(attributes.getValue("type"))) {
				this.dato.setImagenText(true);
			}
		} else if (localName.equals("consultas")) {
			this.consulta = new Consulta();
			this.triples = new ArrayList<Triple>();
			if ("AND".equalsIgnoreCase(attributes.getValue("type"))) {
				this.consulta.setTipoConsultaAND();
			}
			if ("OR".equalsIgnoreCase(attributes.getValue("type"))) {
				this.consulta.setTipoConsultaOR();
			}
			if ("SEARCH".equalsIgnoreCase(attributes.getValue("type"))) {
				this.consulta.setTipoConsultaSEARCH();
			}
		} else if (localName.equals("uri")) {
			this.consulta = new Consulta();
			this.triples = new ArrayList<Triple>();
			if ("AND".equalsIgnoreCase(attributes.getValue("type"))) {
				this.consulta.setTipoConsultaAND();
			}
			if ("OR".equalsIgnoreCase(attributes.getValue("type"))) {
				this.consulta.setTipoConsultaOR();
			}
		} else if (localName.equals("consulta")) {
			this.triple = new Triple();
			if ("objeto".equalsIgnoreCase(attributes.getValue("resultado"))) {
				this.triple.setTipoObjetoConsultadoObjeto();
			} else if ("sujeto".equalsIgnoreCase(attributes.getValue("resultado"))) {
				this.triple.setTipoObjetoConsultadoSujeto();
			} else if ("todos".equalsIgnoreCase(attributes.getValue("resultado"))) {
				this.triple.setTipoObjetoConsultadoTodos();
			}
			if ("objeto".equalsIgnoreCase(attributes.getValue("encadenaAnterior"))) {
				this.triple.setEncadenaAnteriorObjeto();
			} else if ("sujeto".equalsIgnoreCase(attributes.getValue("encadenaAnterior"))) {
				this.triple.setEncadenaAnteriorSujeto();
			}
		} else if (localName.equals("tuberia")) {
			this.consultaTuberia = new ConsultaTuberia();
			this.triplesTuberia = new ArrayList<Triple>();
		} else if (localName.equals("accesible")) {
			this.categoriaAccesible = new Categoria();
		} else if (localName.equals("ficheroconfig")) {
			if (this.categoriaAccesible != null) {
			}
		}
	}

}
