/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.hdtourist.R;
import org.hdtourist.model.Categoria;
import org.hdtourist.model.CategoriaSearch;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.presenter.InformacionListPresenter;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.utils.HdTIdioma;

import android.content.Context;

// TODO: Auto-generated Javadoc
/**
 * The Class InformacionConfigParser.
 */
public class InformacionConfigParser {

	/** The instance. */
	private static InformacionConfigParser INSTANCE = null;

	/** The contexto. */
	private Context contexto;

	// Private constructor suppresses
	/**
	 * Instantiates a new informacion config parser.
	 */
	private InformacionConfigParser() {
		this.contexto = HdTouristService.getInstance().getContexto();
	}

	/**
	 * Gets the single instance of InformacionConfigParser.
	 *
	 * @return single instance of InformacionConfigParser
	 */
	public static InformacionConfigParser getInstance() {
		InformacionConfigParser.createInstance();
		return InformacionConfigParser.INSTANCE;
	}

	/**
	 * Crea el instance.
	 */
	private synchronized static void createInstance() {
		if (InformacionConfigParser.INSTANCE == null) {
			InformacionConfigParser.INSTANCE = new InformacionConfigParser();
		}
	}

	/**
	 * Obtener informacion config.
	 *
	 * @param categoria
	 *            the categoria
	 * @return the informacion list presenter
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public InformacionListPresenter obtenerInformacionConfig(Categoria categoria) throws HdtException {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		String fichero = categoria.getFicheroConf();

		try {
			SAXParser parser = factory.newSAXParser();
			InformacionConfigHandler handler = new InformacionConfigHandler();
			parser.parse(this.getInputStream(fichero), handler);
			InformacionListPresenter informacion = handler.getInformacionConfig();
			if (categoria instanceof CategoriaSearch) {
				informacion.setTexto(((CategoriaSearch) categoria).getTexto());
			}
			return informacion;
		} catch (Exception e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_leyendo_configuracion));
		}
	}

	/**
	 * Gets the input stream.
	 *
	 * @param fichero
	 *            the fichero
	 * @return the input stream
	 */
	private InputStream getInputStream(String fichero) {
		try {
			File fichConf = new File(FicherosService.getInstance().obtenDirectorioConfiguracion().getAbsolutePath() + File.separator + fichero
					+ ".xml");
			if (fichConf.exists()) {
				return new FileInputStream(fichConf);
			} else {
				int id = this.contexto.getResources().getIdentifier(fichero, "raw", this.contexto.getPackageName());
				InputStream fraw = this.contexto.getResources().openRawResource(id);
				return fraw;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
