/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.util.List;

import org.hdtourist.model.Categoria;
import org.hdtourist.model.CategoriaSearch;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.xml.CategoriasConfigParser;
import org.hdtourist.view.CiudadActivity;
import org.hdtourist.view.InformacionFragment;
import org.hdtourist.view.utils.ListaCategoriasAdapter;

import android.os.Bundle;
import android.support.v4.view.MenuItemCompat;

// TODO: Auto-generated Javadoc
/**
 * The Class CiudadPresenter.
 */
public class CiudadPresenter extends CiudadInfoPresenter {

	/** The categoria inicial. */
	private Categoria categoriaInicial;

	/** The lista categorias. */
	private List<Categoria> listaCategorias;

	/**
	 * Inicializa la ventana de informacin de una ciudad Genera el men obteniendo del modelo las categorias.
	 *
	 * @param activity
	 *            the activity
	 * @param bundle
	 *            the bundle
	 */
	public CiudadPresenter(CiudadActivity activity, Bundle bundle) {
		super();
		this.activity = activity;

		this.ciudad = (FicheroHdt) bundle.get("CIUDAD");

		if (bundle.get("CATEGORIA_INICIAL") != null) {
			this.categoriaInicial = (Categoria) bundle.get("CATEGORIA_INICIAL");
		} else {
			this.categoriaInicial = null;
		}

		String titulo = activity.getInformacionFragmentActual() != null ? activity.getInformacionFragmentActual().getTitle() : ciudad.getNombre();
		this.getActivity().setTitle(titulo);

		this.listaCategorias = CategoriasConfigParser.getInstance().obtenerCategorias();
	}

	/**
	 * Gets the fragment.
	 *
	 * @return the fragment
	 */
	public InformacionFragment getFragment() {
		if (this.getActivity().getSearchMenu() != null) {
			if (this.categoriaInicial instanceof CategoriaSearch) {
				this.getActivity().getSearchMenu().setEnabled(false);
			} else {
				this.getActivity().getSearchMenu().setEnabled(true);
			}
		}
		if (this.categoriaInicial != null) {
			return this.getFragment(this.categoriaInicial);
		}
		return this.getFragment(this.listaCategorias.get(0));
	}

	/**
	 * Gets the menu adapter.
	 *
	 * @return the menu adapter
	 */
	public ListaCategoriasAdapter getMenuAdapter() {
		return new ListaCategoriasAdapter(this.listaCategorias, this.getActivity());
	}

	/**
	 * Search.
	 *
	 * @param query
	 *            the query
	 * @return true, if successful
	 */
	public boolean search(String query) {
		query = query.trim();
		CategoriaSearch searchCat = new CategoriaSearch(query);
		searchCat.setNombre("Bsqueda: " + query);
		this.getActivity().replaceFragment(this.getFragment(searchCat), true);
		MenuItemCompat.collapseActionView(this.getActivity().getSearchMenu());
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadInfoPresenter#getActivity()
	 */
	@Override
	protected CiudadActivity getActivity() {
		return this.activity;
	}
}
