/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.io.File;
import java.util.HashMap;
import java.util.List;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.ListaGeoPosiciones;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.InformacionService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.utils.FileUtils;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.CiudadesDisponiblesListFragment;
import org.hdtourist.view.MapaActivity;
import org.hdtourist.view.base.BaseActionBarActivity;
import org.hdtourist.view.base.BaseFragment;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;
import org.hdtourist.view.utils.ListaCiudadesDisponiblesAdapter;
import org.hdtourist.view.utils.NotificationProgress;
import org.hdtourist.view.utils.SpinnerProgressDialog;

import android.app.DownloadManager;
import android.app.DownloadManager.Query;
import android.app.DownloadManager.Request;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Environment;
import android.view.View;

/**
 * Presentador de la arquitectura M-V-P Para la actividad que muestra las ciudades disponibles.
 *
 */
public class CiudadesDisponiblesPresenter extends CiudadesListaPresenter {

	/**
	 * La tarea asncrona EliminarCiudadTask.
	 * Elimina la ciudad pasada por parmetro
	 */
	private class EliminarCiudadTask extends AsyncTask<FicheroHdt, Integer, Boolean> {

		/** El notificador de progreso. */
		private SpinnerProgressDialog notificadorProgreso;

		/** Almacena una excepcin si se produce en la tarea. */
		HdtException excepcion;

		/** El fichero ciudad. */
		FicheroHdt ficheroCiudad;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(FicheroHdt... params) {
			this.ficheroCiudad = params[0];
			try {
				FileUtils.forceDelete(this.ficheroCiudad.getRutaFichero());
				FicherosService.eliminarFicheroDescargado(this.ficheroCiudad);
				return true;
			} catch (HdtException e) {
				this.excepcion = e;
				return false;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
			this.notificadorProgreso.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			this.notificadorProgreso.setFinProceso();
			if (!result) {
				DialogAlerta.mostrarError(this.excepcion);
			} else {
				CiudadesDisponiblesPresenter.this.getFragment().refresh();
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
			this.notificadorProgreso = new SpinnerProgressDialog(CiudadesDisponiblesPresenter.this.getActivity());
			this.notificadorProgreso.setTexto(HdTIdioma.getString(R.string.progreso_eliminado_hdt));
			this.notificadorProgreso.setInicioProceso();
		}

	}

	/**
	 * La tarea asncrona EliminarMapaTask.
	 * Elimina el mapa de la ciudad pasada por parmetro
	 */
	private class EliminarMapaTask extends AsyncTask<FicheroHdt, Integer, Boolean> {

		/** El notificador de progreso. */
		private SpinnerProgressDialog notificadorProgreso;

		/** Almacena una excepcin si se produce en el proceso. */
		HdtException excepcion;

		/** El fichero ciudad. */
		FicheroHdt ficheroCiudad;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(FicheroHdt... params) {
			this.ficheroCiudad = params[0];
			try {
				FileUtils.forceDelete(this.ficheroCiudad.getRutaMapa());
				return true;
			} catch (HdtException e) {
				this.excepcion = e;
				return false;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
			this.notificadorProgreso.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			this.notificadorProgreso.setFinProceso();
			if (!result) {
				DialogAlerta.mostrarError(this.excepcion);
			} else {
				CiudadesDisponiblesPresenter.this.getFragment().refresh();
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
			this.notificadorProgreso = new SpinnerProgressDialog(CiudadesDisponiblesPresenter.this.getActivity());
			this.notificadorProgreso.setTexto(HdTIdioma.getString(R.string.progreso_eliminado_mapa));
			this.notificadorProgreso.setInicioProceso();
		}

	}

	/**
	 * La tarea asncrona ShowCityMapTask.
	 * Calcula los puntos visibles de la ciudad pasada por parmetro y muestra el mapa con las situaciones
	 */
	private class ShowCityMapTask extends AsyncTask<FicheroHdt, Integer, Boolean> {

		/** Almancena una excepcin si se produce en la tarea. */
		private HdtException excepcion;

		/** El fichero de la ciudad. */
		private FicheroHdt ficheroCiudad;

		/** la lista de geoposiciones. */
		private ListaGeoPosiciones listaGeoposiciones = new ListaGeoPosiciones();

		/** The notificador progreso. */
		private SpinnerProgressDialog notificadorProgreso;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(FicheroHdt... params) {
			this.ficheroCiudad = params[0];
			try {
				this.listaGeoposiciones = InformacionService.getInstance().obtenPuntosDelMapa(this.ficheroCiudad);
				return true;
			} catch (HdtException e) {
				this.excepcion = e;
				return false;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
			this.notificadorProgreso.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			this.notificadorProgreso.setFinProceso();
			if (!result) {
				DialogAlerta.mostrarError(this.excepcion);
			} else {
				Intent intent = new Intent(CiudadesDisponiblesPresenter.this.getActivity(), MapaActivity.class);
				intent.putExtra("LISTA", this.listaGeoposiciones);
				CiudadesDisponiblesPresenter.this.getActivity().startActivityForResult(intent, 0000);
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
			this.notificadorProgreso = new SpinnerProgressDialog(CiudadesDisponiblesPresenter.this.getActivity());
			this.notificadorProgreso.setTexto(HdTIdioma.getString(R.string.progreso_obteniendo_puntos_mapa));
			this.notificadorProgreso.setInicioProceso();
		}

	}

	/**
	 * The Class UnzipMaps.
	 */
	private class UnzipMaps extends HdtUnzipTask<String, Boolean> {

		/** The excepcion. */
		private HdtException excepcion;

		/** The nombre ciudad. */
		private String nombreCiudad = "";

		/** The notification progress. */
		private NotificationProgress notificationProgress;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(String... params) {
			String file = params[0];
			String ruta = params[1];
			this.nombreCiudad = params[2];
			try {
				this.unzip(file, ruta);
				FileUtils.forceDelete(file);
				return true;
			} catch (HdtException e) {
				this.excepcion = e;
				return false;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
			this.notificationProgress.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			if (result) {
				if (CiudadesDisponiblesPresenter.this.getFragment() != null && CiudadesDisponiblesPresenter.this.getFragment().isVisible()) {
					CiudadesDisponiblesPresenter.this.getFragment().refresh();
				}
				this.notificationProgress.setTexto(HdTIdioma.getString(R.string.progreso_descarga_mapas_fin, this.nombreCiudad));
				this.notificationProgress.forzarNotificacion();
			} else {
				DialogAlerta.mostrarError(this.excepcion);
			}
			this.notificationProgress.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {

			this.notificationProgress = new NotificationProgress(NotificationProgress.NOTIFICATION_MAPAS_ID, "Obteniedo mapas");
			this.notificationProgress.setNumProcesos(1);
			this.notificationProgress.setProgresoProceso(0);
			this.publishProgress();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(java.lang.Object[])
		 */
		@Override
		protected void onProgressUpdate(Integer... values) {
			if (values.length > 0) {
				this.notificationProgress.setProgresoProceso(values[0]);
			} else {

				this.notificationProgress.setTitulo(HdTIdioma.getString(R.string.progreso_descarga_mapas_title) + this.nombreCiudad);

				this.notificationProgress.setTexto(HdTIdioma.getString(R.string.progreso_descarga_mapas_descomprime));
				this.notificationProgress.setIcono(R.drawable.ic_city);

				this.notificationProgress.forzarNotificacion();

			}
			super.onProgressUpdate(values);
		}

	}

	/** The download manager. */
	private DownloadManager downloadManager;

	/** The mapas de ciudes descargando. */
	private HashMap<Long, FicheroHdt> mapasDeCiudesDescargando = new HashMap<Long, FicheroHdt>();

	/** The map receiver. */
	private BroadcastReceiver mapReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			if (DownloadManager.ACTION_DOWNLOAD_COMPLETE.equals(action)) {
				long downloadId = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, 0);
				if (!CiudadesDisponiblesPresenter.this.mapasDeCiudesDescargando.containsKey(downloadId)) {
					return;
				}
				Query query = new Query();
				query.setFilterById(downloadId);
				Cursor c = CiudadesDisponiblesPresenter.this.downloadManager.query(query);
				if (c.moveToFirst()) {
					int columnIndex = c.getColumnIndex(DownloadManager.COLUMN_STATUS);
					if (DownloadManager.STATUS_SUCCESSFUL == c.getInt(columnIndex)) {

						try {

							FicheroHdt ciudad = CiudadesDisponiblesPresenter.this.mapasDeCiudesDescargando.get(downloadId);
							File direct = new File(Environment.getExternalStorageDirectory() + CiudadesDisponiblesPresenter.this.TEMP_DIR);

							String rutaMapa = ciudad.getRutaMapa();
							File fc = new File(rutaMapa);
							String fileName = fc.getName() + ".zip";
							String fileDescargado = direct.getAbsolutePath() + File.separator + fileName;
							String nombreCiudad = ciudad.getNombre();
							new UnzipMaps().execute(fileDescargado, rutaMapa, nombreCiudad);
						} catch (HdtException e) {
							e.printStackTrace();
						}
					}
				}
				CiudadesDisponiblesPresenter.this.getActivity().unregisterReceiver(CiudadesDisponiblesPresenter.this.mapReceiver);
			}
		}
	};

	/** The temp dir. */
	private String TEMP_DIR = "/temp_hdt";

	/** The fragment. */
	CiudadesDisponiblesListFragment fragment;

	/**
	 * Instantiates a new ciudades disponibles presenter.
	 *
	 * @param fragment
	 *            the fragment
	 */
	public CiudadesDisponiblesPresenter(CiudadesDisponiblesListFragment fragment) {
		this.fragment = fragment;
	}

	/**
	 * Carga ciudades disponibles.
	 *
	 * @return true, if successful
	 */
	public boolean cargaCiudadesDisponibles() {

		try {
			List<FicheroHdt> listado = FicherosService.obtenFicherosHdtDescargados();
			this.fragment.getCiudadesListView().setAdapter(new ListaCiudadesDisponiblesAdapter(this, listado, this.fragment));
			if (listado.size() > 0) {
				this.fragment.getEmptyView().setVisibility(View.INVISIBLE);
				this.fragment.getCiudadesListView().setVisibility(View.VISIBLE);
			} else {
				this.fragment.getEmptyView().setVisibility(View.VISIBLE);
				this.fragment.getCiudadesListView().setVisibility(View.INVISIBLE);
			}
			return true;
		} catch (HdtException e) {
			DialogAlerta.mostrarError(e);
			return false;
		}
	}

	/**
	 * Descargar mapa.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @return true, if successful
	 */
	public boolean descargarMapa(FicheroHdt ciudad) {
		try {
			this.getActivity().registerReceiver(this.mapReceiver, new IntentFilter(DownloadManager.ACTION_DOWNLOAD_COMPLETE));

			String rutaMapa = ciudad.getRutaMapa();
			File fc = new File(rutaMapa);
			String fileName = fc.getName() + ".zip";
			String urlServer = PreferencesService.getMapsUrl() + fileName;
			this.downloadManager = (DownloadManager) this.getActivity().getSystemService(Context.DOWNLOAD_SERVICE);
			Request request = new Request(Uri.parse(urlServer));
			File direct = new File(Environment.getExternalStorageDirectory() + this.TEMP_DIR);
			FileUtils.forceDelete(direct.getAbsolutePath() + File.separator + fileName);
			FicherosService.creaDirSiNoExiste(direct);
			request.setDestinationInExternalPublicDir(this.TEMP_DIR, fileName);
			long id = this.downloadManager.enqueue(request);
			this.mapasDeCiudesDescargando.put(id, ciudad);
			return true;
		} catch (HdtException e) {
			DialogAlerta.mostrarError(e);
			return false;
		}
	}

	/**
	 * Elimina ciudad.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @return true, if successful
	 */
	public boolean eliminaCiudad(FicheroHdt ciudad) {
		new EliminarCiudadTask().execute(ciudad);
		return true;
	}

	/**
	 * Elimina mapa.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @return true, if successful
	 */
	public boolean eliminaMapa(FicheroHdt ciudad) {
		new EliminarMapaTask().execute(ciudad);
		return true;
	}

	/**
	 * Show map.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @return true, if successful
	 */
	public boolean showMap(FicheroHdt ciudad) {
		new ShowCityMapTask().execute(ciudad);
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadesListaPresenter#getActivity()
	 */
	@Override
	protected BaseActionBarActivity getActivity() {
		return (BaseActionBarActivity) this.fragment.getActivity();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadesListaPresenter#getFragment()
	 */
	@Override
	protected BaseFragment getFragment() {
		return this.fragment;
	}
}
