/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.util.ArrayList;

import org.hdtourist.model.FicheroHdt;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.ObtenerCiudadesListFragment;
import org.hdtourist.view.base.BaseActionBarActivity;
import org.hdtourist.view.base.BaseFragment;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.ListaObtenerCiudadesAdapter;

import android.view.View;

/**
 * Presentador de la arquitectura M-V-P Para la actividad que muestra las ciudades en el servidor Maneja la obtencin asncrona de las ciudades y su bsqueda.
 *
 */
public class CiudadesServidorPresenter extends CiudadesListaPresenter {

	/**
	 * Tarea asncrona para obtener las ciudades del servidor.
	 *
	 * @author Elena Hervalejo
	 */
	private class CiudadesServidorTask extends HdtLittleDownloadTask<Void, Boolean> {

		/** Excepcin de la aplicacin */
		private HdtException excep;

		/** The listado. */
		private ArrayList<FicheroHdt> listado = new ArrayList<FicheroHdt>();

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(Void... params) {

			try {
				this.listado = this.obtenFicherosHdtServidor();
				return true;
			} catch (HdtException e1) {
				this.excep = e1;
				return false;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			if (result) {
				CiudadesServidorPresenter.this.setCiudadesServidorList(this.listado);
			} else {
				DialogAlerta.mostrarError(this.excep);
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(java.lang.Object[])
		 */
		@Override
		protected void onProgressUpdate(Integer... values) {
		}
	}

	/** The listado ciudades. */
	private ArrayList<FicheroHdt> listadoCiudades;

	/** The fragment. */
	ObtenerCiudadesListFragment fragment;

	/**
	 * Constructor.
	 *
	 * @param fragment
	 *            the fragment
	 */
	public CiudadesServidorPresenter(ObtenerCiudadesListFragment fragment) {
		this.fragment = fragment;
	}

	/**
	 * Realiza una bsqueda entre las ciudades obtenidas del servidor No se conecta de nuevo al servidor, si no que busca entre las ciudades anteriormente obtenidas.
	 *
	 * @param textoBusqueda
	 *            the texto busqueda
	 * @return <tt>true</tt> si hay resultados
	 */
	public boolean busqueda(String textoBusqueda) {
		ListaObtenerCiudadesAdapter adapter = new ListaObtenerCiudadesAdapter(this, this.listadoCiudades, this.fragment, textoBusqueda);
		this.fragment.getCiudadesListView().setAdapter(adapter);
		if (adapter.haveResults()) {
			this.fragment.getSearchNoResultsTextView().setVisibility(View.GONE);
		} else {
			this.fragment.getSearchNoResultsTextView().setVisibility(View.VISIBLE);
		}
		return adapter.haveResults();
	}

	/**
	 * Tarea para obtener las ciudades del servidor y posteriormente mostrarla.
	 *
	 * @return true, if successful
	 */
	public boolean cargaCiudadesServidor() {
		CiudadesServidorTask ciudadesServidorTask = new CiudadesServidorTask();
		ciudadesServidorTask.execute();
		return true;
	}

	/**
	 * Muestra la lista de ciudades obtenidas (de la bsqueda o del servidor) y gestiona los textos de sin resultados.
	 *
	 * @param listado
	 *            the new ciudades servidor list
	 */
	public void setCiudadesServidorList(ArrayList<FicheroHdt> listado) {
		this.listadoCiudades = listado;
		this.fragment.getCiudadesListView().setAdapter(new ListaObtenerCiudadesAdapter(this, listado, this.fragment, null));
		this.fragment.getListView().setVisibility(View.VISIBLE);
		this.fragment.getProgressView().setVisibility(View.GONE);
		if (listado.size() > 0) {
			this.fragment.getEmptyListTextView().setVisibility(View.GONE);
			this.fragment.getCiudadesListView().setVisibility(View.VISIBLE);
		} else {
			this.fragment.getEmptyListTextView().setVisibility(View.VISIBLE);
			this.fragment.getCiudadesListView().setVisibility(View.GONE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadesListaPresenter#getActivity()
	 */
	@Override
	protected BaseActionBarActivity getActivity() {
		return (BaseActionBarActivity) this.fragment.getActivity();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadesListaPresenter#getFragment()
	 */
	@Override
	protected BaseFragment getFragment() {
		return this.fragment;
	}

}
