/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import org.hdtourist.model.Categoria;
import org.hdtourist.model.CategoriaLugar;
import org.hdtourist.model.CategoriaSearch;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.InformacionService;
import org.hdtourist.model.services.xml.InformacionConfigParser;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.CiudadActivity;
import org.hdtourist.view.InformacionFragment;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.InformacionAdapter;

import android.os.AsyncTask;
import android.support.v7.app.ActionBarActivity;
import android.view.View;

// TODO: Auto-generated Javadoc
/**
 * Presentador de la arquitectura M-V-P Para la gestin de las pantallas de informacin que proviene del fichero HDT. Maneja la tarea asncrona de rellenar la pantalla de informacin obtenida a partir de consultas al hdt.
 *
 * @author Elena Hervalejo
 *
 */
public class InformacionPresenter extends CiudadInfoPresenter {

	/**
	 * Tarea asncrona para obtener la informacin de la categoria actual Al finalizar se pinta en la vista.
	 *
	 * @author Elena Hervalejo
	 */
	private class InformacionTask extends AsyncTask<Void, Integer, InformacionListPresenter> {

		/** The excepcion. */
		private HdtException excepcion = null;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected InformacionListPresenter doInBackground(Void... arg0) {
			try {
				InformacionListPresenter informacion;
				informacion = InformacionConfigParser.getInstance().obtenerInformacionConfig(InformacionPresenter.this.categoria);
				InformacionService.getInstance().informaFicheroHdt(informacion, InformacionPresenter.this.ciudad,
						InformacionPresenter.this.categoria.getUri());
				return informacion;
			} catch (HdtException e) {
				// TODO Auto-generated catch block
				this.excepcion = e;
				return null;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled(java.lang.Object)
		 */
		@Override
		protected void onCancelled(InformacionListPresenter b) {
			super.onCancelled();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(InformacionListPresenter result) {
			if (result != null) {
				InformacionPresenter.this.pintarPantalla(result);
				InformacionPresenter.this.fragment.setCargado(true);
			} else {
				DialogAlerta.mostrarError(this.excepcion);
			}

		}

	}

	/** The instance. */
	private static InformacionPresenter INSTANCE = null;

	/** The categoria. */
	private Categoria categoria;

	/** The fragment. */
	private InformacionFragment fragment;

	/** The informacion task. */
	private InformacionTask informacionTask = null;

	/**
	 * Instantiates a new informacion presenter.
	 *
	 * @param fragment
	 *            the fragment
	 * @param ciudad
	 *            the ciudad
	 * @param categoria
	 *            the categoria
	 */
	private InformacionPresenter(InformacionFragment fragment, FicheroHdt ciudad, Categoria categoria) {
		this.fragment = fragment;
		this.activity = fragment.getCiudadActivity();
		this.ciudad = ciudad;
		this.categoria = categoria;
	}

	/**
	 * Gets the single instance of InformacionPresenter.
	 *
	 * @param fragment
	 *            the fragment
	 * @return single instance of InformacionPresenter
	 */
	public static InformacionPresenter getInstance(InformacionFragment fragment) {
		InformacionPresenter.INSTANCE.fragment = fragment;
		return InformacionPresenter.INSTANCE;
	}

	/**
	 * Gets the single instance of InformacionPresenter.
	 *
	 * @param fragment
	 *            the fragment
	 * @param ciudad
	 *            the ciudad
	 * @param categoria
	 *            the categoria
	 * @return single instance of InformacionPresenter
	 */
	public static InformacionPresenter getInstance(InformacionFragment fragment, FicheroHdt ciudad, Categoria categoria) {
		InformacionPresenter.INSTANCE = new InformacionPresenter(fragment, ciudad, categoria);
		return InformacionPresenter.INSTANCE;
	}

	/**
	 * Ejecuta la tarea asncrona para obtener la informacin de la categora actual.
	 */
	public void cargaInformacion() {
		this.informacionTask = new InformacionTask();
		this.informacionTask.execute();
	}

	/**
	 * Mtodo para evitar fallos cuando se sale de la pantalla antes de que haya terminado la tarea asncrona de obtencin de informacin.
	 */
	public void destroy() {
		this.informacionTask.cancel(true);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.presenter.CiudadInfoPresenter#getActivity()
	 */
	@Override
	public CiudadActivity getActivity() {
		return (CiudadActivity) this.fragment.getActivity();
	}

	/**
	 * Obtiene la categora de informacin actual.
	 *
	 * @return the categoria
	 */
	public Categoria getCategoria() {
		return this.categoria;
	}

	/**
	 * Gets the title.
	 *
	 * @return the title
	 */
	public String getTitle() {
		if (this.isSearch()) {
			return this.categoria.getNombre();
		}
		if (this.getActivity() != null && !this.getActivity().hayFragmentsEnLaPila()) {
			return this.categoria.getNombre();
		} else {
			return this.getCiudad().getNombre();
		}
	}

	/**
	 * Indica si actualmente estamos en una bsqueda de informacin hdt.
	 *
	 * @return true, if is search
	 */
	public boolean isSearch() {
		return this.categoria instanceof CategoriaSearch;
	}

	/**
	 * Mostrar situacion.
	 *
	 * @return true, if successful
	 */
	public boolean mostrarSituacion() {
		if (this.categoria instanceof CategoriaLugar) {
			return false;
		}
		return true;
	}

	/**
	 * Mtodo que se ejecuta una vez obtenida la informacin de forma asncrona para pintar dicha informacin en la vista.
	 *
	 * @param informacion
	 *            the informacion
	 */
	public void pintarPantalla(InformacionListPresenter informacion) {
		this.fragment.setListAdapter(new InformacionAdapter(informacion, this.fragment));
		if (informacion != null && informacion.getListaInformacion() != null && informacion.getListaInformacion().size() > 0) {
			this.fragment.getProgressBar().setVisibility(View.INVISIBLE);
			this.fragment.getTextEmptyView().setVisibility(View.INVISIBLE);
		} else {
			this.fragment.getProgressBar().setVisibility(View.INVISIBLE);
			this.fragment.getTextEmptyView().setVisibility(View.VISIBLE);
		}
		((ActionBarActivity) this.getActivity()).getSupportActionBar().setTitle(this.getTitle());
	}
}
