/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.io.File;

import org.hdtourist.R;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.utils.Constantes;
import org.hdtourist.utils.FileUtils;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.CiudadesDisponiblesActivity;
import org.hdtourist.view.InicioActivity;
import org.hdtourist.view.utils.DialogAlerta;

import android.content.Intent;
import android.os.AsyncTask;

// TODO: Auto-generated Javadoc
/**
 * Presentador de la arquitectura M-V-P Para la actividad de inicio Se encarga de la configuracin inicial Se establece un tiempo mnimo para mostrar el splash de bienvendia.
 *
 * @author Elena Hervalejo
 */
public class InicioPresenter {

	/**
	 * Tarea asncrona de configuracin inicial Una vez terminada y pasado el tiempo mnimo, se dirige inicia la actividad inicial de la aplicacin.
	 *
	 * @author Elena Hervalejo
	 */
	private class InicioTask extends AsyncTask<Void, Integer, Boolean> {

		/** The excep. */
		HdtException excep;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(Void... params) {
			long starttime = System.currentTimeMillis();

			try {
				// Copiamos los archivos de configuracin al dispositivo.
				File dirConfig = FicherosService.getInstance().obtenDirectorioConfiguracion();
				for (String ficheroConf : Constantes.FICHEROS_CONFIGURACION) {
					File fichero = new File(dirConfig.getAbsolutePath() + File.separator + ficheroConf);
					if (!fichero.exists()) {
						int id = InicioPresenter.this.activity.getResources().getIdentifier(FileUtils.getBaseFileName(fichero), "raw",
								InicioPresenter.this.activity.getPackageName());
						FicherosService.copyRAWtoSDCard(id, fichero.getAbsolutePath());
					}
				}

				long endtime;
				long tiempo_min = InicioPresenter.this.activity.getResources().getInteger(R.integer.const_tiempo_pantalla_inicio);
				endtime = System.currentTimeMillis();
				if (endtime - starttime < tiempo_min) {
					try {
						Thread.sleep(tiempo_min);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			} catch (HdtException e1) {
				this.excep = e1;
				return false;
			}
			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			if (result) {
				Intent mainIntent = new Intent(InicioPresenter.this.activity, CiudadesDisponiblesActivity.class);
				InicioPresenter.this.activity.startActivity(mainIntent);
				InicioPresenter.this.activity.finish();
			} else {
				DialogAlerta.mostrarError(this.excep);
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(java.lang.Object[])
		 */
		@Override
		protected void onProgressUpdate(Integer... values) {
		}
	}

	/** The activity. */
	InicioActivity activity;

	/**
	 * Instantiates a new inicio presenter.
	 *
	 * @param activity
	 *            the activity
	 */
	public InicioPresenter(InicioActivity activity) {
		this.activity = activity;
	}

	/**
	 * Ejecuta la tarea asncrona de configuracin inicial.
	 *
	 * @return true, if successful
	 */
	public boolean cargaInicio() {
		InicioTask inicioTask = new InicioTask();
		inicioTask.execute();
		return true;
	}

}
