/*
 * @author Elena Hervalejo
 */
package org.hdtourist.utils;

import java.io.File;

/**
 * Clase de utilidades para manejar ficheros y directorios del disco duro.
 */
public class FileUtils {

	/**
	 * Borra el directorio y todo su contenido.
	 *
	 * @param path
	 *            the path
	 * @return true, si se realiza correctamente
	 */
	public static boolean forceDelete(String path) {
		File file = new File(path);
		if (!file.exists()) {
			return true;
		}
		if (!file.isDirectory()) {
			return file.delete();
		}
		return FileUtils.deleteChildren(file) && file.delete();
	}

	/**
	 * Obtiene el nombre del fichero sin la extensin
	 *
	 * @param fichero
	 *            el fichero
	 * @return el nombre del fichero sin extensin
	 */
	public static String getBaseFileName(File fichero) {
		String name = fichero.getName();
		int pos = name.lastIndexOf(".");
		if (pos > 0) {
			name = name.substring(0, pos);
		}
		return name;
	}

	/**
	 * Elimina el directorio y sus archivos.
	 *
	 * @param dir
	 *            el directorio
	 * @return true, si se borra correctamente
	 */
	private static boolean deleteChildren(File dir) {
		File[] children = dir.listFiles();
		boolean childrenDeleted = true;
		for (int i = 0; children != null && i < children.length; i++) {
			File child = children[i];
			if (child.isDirectory()) {
				childrenDeleted = FileUtils.deleteChildren(child) && childrenDeleted;
			}
			if (child.exists()) {
				childrenDeleted = child.delete() && childrenDeleted;
			}
		}
		return childrenDeleted;
	}
}