/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import java.util.Stack;

import org.hdtourist.R;
import org.hdtourist.model.Categoria;
import org.hdtourist.model.CategoriaSearch;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.presenter.CiudadPresenter;
import org.hdtourist.view.base.BaseMenuActionBarActivity;

import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.MenuItemCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.widget.SearchView;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;
import android.widget.Toast;

/**
 * Activity para mostrar la informacin de la ciudad.
 */
public class CiudadActivity extends BaseMenuActionBarActivity {

	/** The drawer layout. */
	private DrawerLayout drawerLayout;

	/** The drawer list. */
	private ListView drawerList;

	/** The drawer toggle. */
	private ActionBarDrawerToggle drawerToggle;

	/** La clase presenter que gestiona esta actividad. */
	private CiudadPresenter presenter;

	/** La bsqueda menu item. */
	private MenuItem searchMenuItem;

	/** Pila de fragments de informacin guardados a los que volver. */
	private Stack<InformacionFragment> stakFragments = new Stack<InformacionFragment>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getActionsMenu(android.view.MenuItem)
	 */
	@Override
	public boolean getActionsMenu(MenuItem item) {
		if (this.drawerToggle.onOptionsItemSelected(item)) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean getActionsSubMenu(MenuItem item) {
		if (item.getItemId() ==  R.id.menu_informacion_hdt) {

			Intent intent = new Intent(this, InfoHDTActivity.class);
			intent.putExtra("HDT", presenter.getCiudad());
			this.startActivityForResult(intent, 0000);
			return true;
		}
		return super.getActionsSubMenu(item);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getMenu()
	 */
	@Override
	public int getMenu() {
		return R.menu.menu_informacion;
	}

	/**
	 * Obtiene el men de bsqueda almacenado como atributo de la clase
	 *
	 * @return the search menu
	 */
	public MenuItem getSearchMenu() {
		return this.searchMenuItem;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getSubMenu()
	 */
	@Override
	public int getSubMenu() {
		return R.menu.submenu_informacion;
	}

	/**
	 * Indica si hay fragments en la pila.
	 *
	 * @return true, si hay fragments de informacin en la pila, es decir, si hay pantallas de informacin a las que volver
	 */
	public boolean hayFragmentsEnLaPila() {
		return this.stakFragments.isEmpty();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v7.app.ActionBarActivity#onConfigurationChanged(android.content.res.Configuration)
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		this.drawerToggle.onConfigurationChanged(newConfig);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);

		this.searchMenuItem = menu.findItem(R.id.action_search);
		SearchView searchViewAction = (SearchView) MenuItemCompat.getActionView(this.searchMenuItem);
		searchViewAction.setIconifiedByDefault(false);

		searchViewAction.setOnQueryTextListener(new SearchView.OnQueryTextListener() {

			@Override
			public boolean onQueryTextChange(String newText) {
				// TODO : newText is the text from the search view(event
				// triggered every time the text changes)
				return false;
			}

			@Override
			public boolean onQueryTextSubmit(String query) {
				final int max = 3;
				if (query.length() > max) {
					CiudadActivity.this.presenter.search(query);
					return true;
				} else {
					Toast.makeText(CiudadActivity.this, "Debes introducir ms de " + max + " carcteres", Toast.LENGTH_LONG).show();
				}
				return true;
			}

		});
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.FragmentActivity#onKeyDown(int, android.view.KeyEvent)
	 */
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {

		if (keyCode == KeyEvent.KEYCODE_BACK) {

			if (!this.stakFragments.isEmpty()) {
				InformacionFragment f = this.stakFragments.pop();
				this.replaceFragment(f, false);
				return true;
			}
		}
		// para las demas cosas, se reenvia el evento al listener habitual
		return super.onKeyDown(keyCode, event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onPrepareOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {

		this.drawerLayout.isDrawerOpen(this.drawerList);

		return super.onPrepareOptionsMenu(menu);
	}

	/**
	 * Reemplaza el fragment de informacin que se muestra
	 *
	 * @param fragment
	 *            el nuevo fragment
	 * @param guardarActual
	 *            true si se debe guardar el fragment anterior en la pila para poder volver a el al pulsar el botn "atrs"
	 */
	public void replaceFragment(InformacionFragment fragment, boolean guardarActual) {

		FragmentManager fragmentManager = this.getSupportFragmentManager();
		if (guardarActual && fragmentManager.getFragments().size() > 0) {
			InformacionFragment f = this.getInformacionFragmentActual();
			this.stakFragments.push(f);
		}

		if (this.getSearchMenu() != null) {
			if (fragment.getCategoria() instanceof CategoriaSearch) {
				this.getSearchMenu().setEnabled(false);
			} else {
				this.getSearchMenu().setEnabled(true);
			}
		}

		this.setTitle(fragment.getTitle());

		fragmentManager.beginTransaction().replace(R.id.content_frame, fragment).commit();

		this.drawerLayout.closeDrawer(this.drawerList);

		CiudadActivity.this.drawerLayout.closeDrawers();
	}
	
	@Override
	public void setTitle(CharSequence title) {
		this.getSupportActionBar().setTitle(title);
		super.setTitle(title);
	}
	
	/**
	 * Almacena el men de bsqueda en un atributo de la clase
	 *
	 * @param searchMenuItem
	 *            the searchMenuItem to set
	 */
	public void setSearchMenuItem(MenuItem searchMenuItem) {
		this.searchMenuItem = searchMenuItem;
	}

	/**
	 * Obtiene el fragment de informacin que se est mostrando en este momento.
	 *
	 * @return el fragment de informacin que est visible
	 */
	public InformacionFragment getInformacionFragmentActual() {
		if (this.getSupportFragmentManager().getFragments() == null) {
			return null;
		}
		return (InformacionFragment) this.getSupportFragmentManager().getFragments().get(0);
	}

	/**
	 * Fija el navigation drawer.
	 */
	private void setNavigationDrawer() {

		this.drawerLayout = (DrawerLayout) this.findViewById(R.id.drawer_layout);
		this.drawerList = (ListView) this.findViewById(R.id.left_drawer);

		this.drawerList.setAdapter(this.presenter.getMenuAdapter());

		this.drawerList.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
				// TODO - Para seguir la arquitectura extrictamente no se debera llamar al modelo
				Categoria elegido = (Categoria) parent.getItemAtPosition(position);
				CiudadActivity.this.stakFragments.clear();
				CiudadActivity.this.replaceFragment(CiudadActivity.this.presenter.getFragment(elegido), false);

				CiudadActivity.this.drawerList.setItemChecked(position, true);
			}
		});

		this.drawerToggle = new ActionBarDrawerToggle(this, this.drawerLayout, R.drawable.ic_drawer, R.drawable.ic_drawer, R.string.app_name) {

			@Override
			public void onDrawerClosed(View view) {
				CiudadActivity.this.getSupportActionBar().setTitle(CiudadActivity.this.getInformacionFragmentActual().getTitle());
				ActivityCompat.invalidateOptionsMenu(CiudadActivity.this);
			}

			@Override
			public void onDrawerOpened(View drawerView) {
				CiudadActivity.this.getSupportActionBar().setTitle("Categoras de informacin");
				ActivityCompat.invalidateOptionsMenu(CiudadActivity.this);
			}
		};

		this.drawerLayout.setDrawerListener(this.drawerToggle);

		this.getSupportActionBar().setDisplayHomeAsUpEnabled(true);
		this.getSupportActionBar().setHomeButtonEnabled(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseActionBarActivity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		this.presenter = new CiudadPresenter(this, this.getIntent().getExtras());
		HdTouristService.getInstance(this);

		this.setContentView(R.layout.activity_ciudad);
		this.setNavigationDrawer();

		this.replaceFragment(this.presenter.getFragment(), false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onPostCreate(android.os.Bundle)
	 */
	@Override
	protected void onPostCreate(Bundle savedInstanceState) {
		super.onPostCreate(savedInstanceState);
		this.drawerToggle.syncState();
	}
}
