/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import org.hdtourist.R;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.base.BaseFragment;
import org.hdtourist.view.base.BaseMenuActionBarActivity;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;

import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.view.MenuItem;

import com.nostra13.universalimageloader.cache.disc.impl.UnlimitedDiscCache;
import com.nostra13.universalimageloader.cache.memory.impl.WeakMemoryCache;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

/**
 * Activity para mostrar la lista de ciudades ya descargadas y disponibles en el dispositivo.
 */
public class CiudadesDisponiblesActivity extends BaseMenuActionBarActivity {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getActionsMenu(android.view.MenuItem)
	 */
	@Override
	public boolean getActionsMenu(MenuItem item) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getActionsSubMenu(android.view.MenuItem)
	 */
	@Override
	public boolean getActionsSubMenu(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.action_city_add:
			Intent intent = new Intent(this, ObtenerCiudadesActivity.class);
			if (PreferencesService.isConexionPreguntar()) {
				DialogAlerta.mostrarConfirmacionPantalla(this, HdTIdioma.getString(R.string.mensaje_aviso_requiere_internet_title),
						HdTIdioma.getString(R.string.mensaje_aviso_acceso_obtener_ciudad), intent).show();
			} else if (PreferencesService.isConexionSiempre()) {
				this.getDialogSpinerProgress().setInicioProceso();
				this.getDialogSpinerProgress().setTitulo("");
				this.getDialogSpinerProgress().setTexto(HdTIdioma.getString(R.string.progreso_obteniendo_ciudades_servidor));
				this.startActivityForResult(intent, 00001);
			} else if (PreferencesService.isConexionNunca()) {
				DialogAlerta.mostrarAvisoPreferencias(this, HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada_title),
						HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada_acceso_obtener_ciudad)).show();
			}
			return true;
		}
		return super.getActionsSubMenu(item);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getMenu()
	 */
	@Override
	public int getMenu() {
		return R.menu.menu_ciudades_disponibles;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseMenuActionBarActivity#getSubMenu()
	 */
	@Override
	public int getSubMenu() {
		return R.menu.submenu_ciudades_disponibles;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.FragmentActivity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		((BaseFragment) this.getSupportFragmentManager().getFragments().get(0)).refresh();
		this.getDialogSpinerProgress().setFinProceso();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see android.support.v7.app.ActionBarActivity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);

		DisplayImageOptions defaultOptions = new DisplayImageOptions.Builder().cacheInMemory(true).cacheOnDisc(true)
				.bitmapConfig(Bitmap.Config.RGB_565).showImageOnLoading(R.drawable.ic_city) // resource or drawable
				.showImageForEmptyUri(R.drawable.ic_city) // resource or drawable
				.showImageOnFail(R.drawable.ic_city).build();
		ImageLoaderConfiguration config;
		try {
			config = new ImageLoaderConfiguration.Builder(this).defaultDisplayImageOptions(defaultOptions)
					.discCache(new UnlimitedDiscCache(FicherosService.getInstance().obtenDirectorioImagen())).memoryCache(new WeakMemoryCache())
					.build();
		} catch (HdtException e) {
			config = new ImageLoaderConfiguration.Builder(this).defaultDisplayImageOptions(defaultOptions).memoryCache(new WeakMemoryCache()).build();
		}
		ImageLoader.getInstance().init(config);

		this.setContentView(R.layout.activity_ciudades_disponibles);
		this.getSupportFragmentManager().beginTransaction().add(R.id.itemListFragment, new CiudadesDisponiblesListFragment()).commit();

		HdTouristService.getInstance(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseActionBarActivity#onResume()
	 */
	@Override
	protected void onResume() {
		super.onResume();
		((BaseFragment) this.getSupportFragmentManager().getFragments().get(0)).refresh();
		this.getDialogSpinerProgress().setFinProceso();
	}
}
