/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;

import org.hdtourist.R;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.utils.HdTIdioma;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.text.InputType;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

// TODO: Auto-generated Javadoc
/**
 * Activity para seleccionar un directorio.
 */
public class DirectoryChooserActivity extends ListActivity {

	/** La constante CHOSEN_DIRECTORY. */
	public static final String CHOSEN_DIRECTORY = "chosenDir";

	/** La constante ONLY_DIRS. */
	public static final String ONLY_DIRS = "onlyDirs";

	/** La constante PICK_DIRECTORY. */
	public static final int PICK_DIRECTORY = 49834;

	/** La constante SHOW_HIDDEN. */
	public static final String SHOW_HIDDEN = "showHidden";

	/** La constante START_DIR. */
	public static final String START_DIR = "startDir";

	/** The dir. */
	private File dir;

	/** The only dirs. */
	private boolean onlyDirs = true;

	/** The show hidden. */
	private boolean showHidden = false;

	/**
	 * Filter.
	 *
	 * @param file_list
	 *            the file_list
	 * @param onlyDirs
	 *            the only dirs
	 * @param showHidden
	 *            the show hidden
	 * @return the array list
	 */
	public ArrayList<File> filter(File[] file_list, boolean onlyDirs, boolean showHidden) {
		ArrayList<File> files = new ArrayList<File>();
		for (File file : file_list) {
			if (onlyDirs && !file.isDirectory()) {
				continue;
			}
			if (!showHidden && file.isHidden()) {
				continue;
			}
			files.add(file);
		}
		Collections.sort(files);
		return files;
	}

	/**
	 * Names.
	 *
	 * @param files
	 *            the files
	 * @return the string[]
	 */
	public String[] names(ArrayList<File> files) {
		String[] names = new String[files.size()];
		int i = 0;
		for (File file : files) {
			names[i] = file.getName();
			i++;
		}
		return names;
	}

	/**
	 * New directory name.
	 */
	public void newDirectoryName() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(HdTIdioma.getString(R.string.elegir_directorio_add_name_text));

		// Set up the input
		final EditText input = new EditText(this);
		input.setInputType(InputType.TYPE_CLASS_TEXT);
		builder.setView(input);

		// Set up the buttons
		builder.setPositiveButton(HdTIdioma.getString(R.string.dialog_ok_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				String newDirectoryName = input.getText().toString();
				if (StringUtils.isEmpty(newDirectoryName)) {
					return;
				}
				File newDir = new File(DirectoryChooserActivity.this.dir.getAbsolutePath() + File.separator + newDirectoryName);
				newDir.mkdir();
				String path = newDir.getAbsolutePath();
				Intent intent = new Intent(DirectoryChooserActivity.this, DirectoryChooserActivity.class);
				intent.putExtra(DirectoryChooserActivity.START_DIR, path);
				intent.putExtra(DirectoryChooserActivity.SHOW_HIDDEN, DirectoryChooserActivity.this.showHidden);
				intent.putExtra(DirectoryChooserActivity.ONLY_DIRS, DirectoryChooserActivity.this.onlyDirs);
				DirectoryChooserActivity.this.startActivityForResult(intent, DirectoryChooserActivity.PICK_DIRECTORY);
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_cancelar_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});

		builder.show();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Bundle extras = this.getIntent().getExtras();
		this.dir = Environment.getExternalStorageDirectory();
		if (extras != null) {
			String preferredStartDir = extras.getString(DirectoryChooserActivity.START_DIR);
			this.showHidden = extras.getBoolean(DirectoryChooserActivity.SHOW_HIDDEN, false);
			this.onlyDirs = extras.getBoolean(DirectoryChooserActivity.ONLY_DIRS, true);
			if (preferredStartDir != null) {
				File startDir = new File(preferredStartDir);
				if (startDir.isDirectory()) {
					this.dir = startDir;
				}
			}
		}

		this.setContentView(R.layout.activity_directory_chooser);
		this.setTitle(this.dir.getAbsolutePath());
		TextView textoDir = (TextView) this.findViewById(R.id.act_directory_chooser_dir_actual);
		textoDir.setText(this.dir.getAbsolutePath());
		Button btnChoose = (Button) this.findViewById(R.id.act_directory_chooser_button);
		btnChoose.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				DirectoryChooserActivity.this.returnDir(DirectoryChooserActivity.this.dir.getAbsolutePath());
			}
		});

		ListView lv = this.getListView();
		lv.setTextFilterEnabled(true);

		if (!this.dir.canRead()) {
			Context context = this.getApplicationContext();
			String msg = "Could not read folder contents.";
			Toast toast = Toast.makeText(context, msg, Toast.LENGTH_LONG);
			toast.show();
			return;
		}

		final ArrayList<File> files = this.filter(this.dir.listFiles(), this.onlyDirs, this.showHidden);
		String[] names = this.names(files);
		this.setListAdapter(new ArrayAdapter<String>(this, R.layout.item_directory, names));

		lv.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				if (!files.get(position).isDirectory()) {
					return;
				}
				String path = files.get(position).getAbsolutePath();
				Intent intent = new Intent(DirectoryChooserActivity.this, DirectoryChooserActivity.class);
				intent.putExtra(DirectoryChooserActivity.START_DIR, path);
				intent.putExtra(DirectoryChooserActivity.SHOW_HIDDEN, DirectoryChooserActivity.this.showHidden);
				intent.putExtra(DirectoryChooserActivity.ONLY_DIRS, DirectoryChooserActivity.this.onlyDirs);
				DirectoryChooserActivity.this.startActivityForResult(intent, DirectoryChooserActivity.PICK_DIRECTORY);
			}
		});

		ImageView previousFolder = (ImageView) this.findViewById(R.id.act_directory_chooser_previous_button);
		if (this.dir.getAbsolutePath().equals("/")) {
			previousFolder.setVisibility(View.GONE);
		} else {
			previousFolder.setVisibility(View.VISIBLE);
		}
		previousFolder.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (DirectoryChooserActivity.this.dir.getAbsolutePath().equals("/")) {
					return;
				}
				String path = DirectoryChooserActivity.this.dir.getParentFile().getAbsolutePath();
				Intent intent = new Intent(DirectoryChooserActivity.this, DirectoryChooserActivity.class);
				intent.putExtra(DirectoryChooserActivity.START_DIR, path);
				intent.putExtra(DirectoryChooserActivity.SHOW_HIDDEN, DirectoryChooserActivity.this.showHidden);
				intent.putExtra(DirectoryChooserActivity.ONLY_DIRS, DirectoryChooserActivity.this.onlyDirs);
				DirectoryChooserActivity.this.startActivityForResult(intent, DirectoryChooserActivity.PICK_DIRECTORY);

			}
		});

		ImageView addFolder = (ImageView) this.findViewById(R.id.act_directory_chooser_add_button);
		addFolder.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				DirectoryChooserActivity.this.newDirectoryName();
			}
		});
	}

	/**
	 * Return dir.
	 *
	 * @param path
	 *            the path
	 */
	private void returnDir(String path) {
		Intent result = new Intent();
		result.putExtra(DirectoryChooserActivity.CHOSEN_DIRECTORY, path);
		this.setResult(Activity.RESULT_OK, result);
		this.finish();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == DirectoryChooserActivity.PICK_DIRECTORY && resultCode == Activity.RESULT_OK) {
			Bundle extras = data.getExtras();
			String path = (String) extras.get(DirectoryChooserActivity.CHOSEN_DIRECTORY);
			this.returnDir(path);
		}
	}
}
