/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import org.hdtourist.R;
import org.hdtourist.model.Categoria;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.presenter.InformacionPresenter;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.base.BaseListFragment;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.nostra13.universalimageloader.cache.disc.impl.UnlimitedDiscCache;
import com.nostra13.universalimageloader.cache.memory.impl.WeakMemoryCache;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

/**
 * Fragment para mostrar informacin de una categora de una ciudad.
 */
@SuppressLint("ValidFragment")
public class InformacionFragment extends BaseListFragment {

	/** The inflater. */
	private LayoutInflater inflater;

	/** The presenter. */
	private InformacionPresenter presenter;

	/** The progress bar. */
	private ProgressBar progressBar;

	/** The text empty view. */
	private TextView textEmptyView;

	/** The cargado. */
	boolean cargado = false;

	/**
	 * Instantiates a new informacion fragment.
	 */
	public InformacionFragment() {
		super();
	}

	/**
	 * Instantiates a new informacion fragment.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @param categoria
	 *            the categoria
	 */
	public InformacionFragment(FicheroHdt ciudad, Categoria categoria) {
		this.presenter = InformacionPresenter.getInstance(this, ciudad, categoria);
		new InformacionFragment();
	}

	/**
	 * Gets the categoria.
	 *
	 * @return the categoria
	 */
	public Categoria getCategoria() {
		return this.presenter.getCategoria();
	}

	/**
	 * Gets the ciudad activity.
	 *
	 * @return the ciudad activity
	 */
	public CiudadActivity getCiudadActivity() {
		return (CiudadActivity) this.getActivity();
	}

	/**
	 * Gets the inflater.
	 *
	 * @return the inflater
	 */
	public LayoutInflater getInflater() {
		return this.inflater;
	}

	/**
	 * Gets the progress bar.
	 *
	 * @return the progress bar
	 */
	public ProgressBar getProgressBar() {
		return this.progressBar;
	}

	/**
	 * Gets the text empty view.
	 *
	 * @return the text empty view
	 */
	public TextView getTextEmptyView() {
		return this.textEmptyView;
	}

	/**
	 * Obtiene el ttulo que debe mostrar el activity segn el fragment que muestra
	 *
	 * @return el ttulo
	 */
	public String getTitle() {
		return this.presenter.getTitle();
	}

	/**
	 * Checks if is cargado.
	 *
	 * @return the cargado
	 */
	public boolean isCargado() {
		return this.cargado;
	}

	/**
	 * Indica si este fragment muestra el resultado de una bsqueda.
	 *
	 * @return true, if is search
	 */
	public boolean isSearch() {
		return this.presenter.isSearch();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		DisplayImageOptions defaultOptions = new DisplayImageOptions.Builder().cacheInMemory(true).cacheOnDisc(true)
				.bitmapConfig(Bitmap.Config.RGB_565).showImageOnLoading(R.drawable.ic_image_loading) // resource or drawable
				.showImageForEmptyUri(R.drawable.ic_no_image) // resource or drawable
				.showImageOnFail(R.drawable.ic_no_image).build();
		ImageLoaderConfiguration config;
		try {
			config = new ImageLoaderConfiguration.Builder(this.getActivity()).defaultDisplayImageOptions(defaultOptions)
					.discCache(new UnlimitedDiscCache(FicherosService.getInstance().obtenDirectorioImagen())).memoryCache(new WeakMemoryCache())
					.build();
		} catch (HdtException e) {
			config = new ImageLoaderConfiguration.Builder(this.getActivity()).defaultDisplayImageOptions(defaultOptions)
					.memoryCache(new WeakMemoryCache()).build();
		}
		ImageLoader.getInstance().init(config);

		super.onCreateView(inflater, container, savedInstanceState);
		this.inflater = inflater;
		return inflater.inflate(R.layout.activity_informacion, container, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.Fragment#onDestroy()
	 */
	@Override
	public void onDestroy() {
		this.presenter.destroy();
		super.onDestroy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.ListFragment#onViewCreated(android.view.View, android.os.Bundle)
	 */
	@Override
	public void onViewCreated(View view, Bundle savedInstanceState) {
		super.onViewCreated(view, savedInstanceState);

		this.textEmptyView = (TextView) view.findViewById(android.R.id.empty);
		this.progressBar = (ProgressBar) view.findViewById(R.id.progressBarInformacion);
		this.textEmptyView.setVisibility(View.GONE);
		if (!this.isCargado()) {
			this.presenter.cargaInformacion();
		} else {
			this.progressBar.setVisibility(View.GONE);
		}
	}

	/**
	 * Sets the cargado.
	 *
	 * @param cargado
	 *            the cargado to set
	 */
	public void setCargado(boolean cargado) {
		this.cargado = cargado;
	}

	/**
	 * Sets the inflater.
	 *
	 * @param inflater
	 *            the new inflater
	 */
	public void setInflater(LayoutInflater inflater) {
		this.inflater = inflater;
	}
}
