/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import java.util.ArrayList;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.presenter.CiudadesServidorPresenter;
import org.hdtourist.view.base.BaseFragment;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Fragment que contiene la lista de ciudades que se pueden obtener del servidor.
 */
public class ObtenerCiudadesListFragment extends BaseFragment {

	/** The ciudades list view. */
	private ListView ciudadesListView;

	/** The empty list text view. */
	private TextView emptyListTextView;

	/** The inflater. */
	private LayoutInflater inflater;

	/** The list view. */
	private View listView;

	/** The presenter. */
	private CiudadesServidorPresenter presenter;

	/** The progress view. */
	private View progressView;

	/** The search no results text view. */
	private TextView searchNoResultsTextView;

	/**
	 * Busqueda.
	 *
	 * @param query
	 *            the query
	 */
	public void busqueda(String query) {
		this.presenter.busqueda(query);
	}

	/**
	 * Gets the ciudades list view.
	 *
	 * @return the ciudades list view
	 */
	public ListView getCiudadesListView() {
		return this.ciudadesListView;
	}

	/**
	 * Gets the empty list text view.
	 *
	 * @return the empty list text view
	 */
	public TextView getEmptyListTextView() {
		return this.emptyListTextView;
	}

	/**
	 * Gets the inflater.
	 *
	 * @return the inflater
	 */
	public LayoutInflater getInflater() {
		return this.inflater;
	}

	/**
	 * Gets the list view.
	 *
	 * @return the list view
	 */
	public View getListView() {
		return this.listView;
	}

	/**
	 * Gets the progress view.
	 *
	 * @return the progress view
	 */
	public View getProgressView() {
		return this.progressView;
	}

	/**
	 * Gets the search no results text view.
	 *
	 * @return the search no results text view
	 */
	public TextView getSearchNoResultsTextView() {
		return this.searchNoResultsTextView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.Fragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		super.onCreateView(inflater, container, savedInstanceState);

		this.presenter = new CiudadesServidorPresenter(this);

		this.inflater = inflater;
		View v = inflater.inflate(R.layout.fragment_obtener_ciudades, container, false);
		this.ciudadesListView = (ListView) v.findViewById(R.id.act_obtener_ciudades_list);
		this.emptyListTextView = (TextView) v.findViewById(R.id.act_obtener_ciudades_empty_text);
		this.searchNoResultsTextView = (TextView) v.findViewById(R.id.act_obtener_ciudades_search_no_results);
		this.listView = v.findViewById(R.id.act_obtener_ciudades_listado_view);
		this.progressView = v.findViewById(R.id.act_obtener_ciudades_progress_view);

		this.listView.setVisibility(View.GONE);
		this.searchNoResultsTextView.setVisibility(View.GONE);
		this.progressView.setVisibility(View.VISIBLE);

		return v;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.Fragment#onViewCreated(android.view.View, android.os.Bundle)
	 */
	@Override
	public void onViewCreated(View view, Bundle savedInstanceState) {
		super.onViewCreated(view, savedInstanceState);
		if (!HdTouristService.getInstance().isNetworkAvailable()) {
			DialogAlerta.mostrarAviso(this.getActivity(), HdTIdioma.getString(R.string.mensaje_error_no_conexion_title),
					HdTIdioma.getString(R.string.mensaje_error_no_conexion)).show();
			this.presenter.setCiudadesServidorList(new ArrayList<FicheroHdt>());
			return;
		}
		this.presenter.cargaCiudadesServidor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseFragment#refresh()
	 */
	@Override
	public void refresh() {
		// TODO Auto-generated method stub

	}
}