/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import java.io.File;

import org.hdtourist.R;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.utils.Constantes;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;

/**
 * Activity para configurar la aplicacin.
 */
@SuppressLint("NewApi")
public class OpcionesActivity extends PreferenceActivity {

	/**
	 * La tarea asncrona CambioDirectorioTask.
	 * Tarea de cambio de directorio
	 */
	private class CambioDirectorioTask extends AsyncTask<String, Integer, Boolean> {

		/** El antiguo path. */
		String antiguoPath;

		/** Almacena la excepcin si se produce al realizar la tarea. */
		HdtException excep = null;

		/** El nuevo path. */
		String nuevoPath;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(String... params) {
			this.antiguoPath = params[0];
			this.nuevoPath = params[1];
			try {
				FicherosService.moverArchivosConfiguracion(this.antiguoPath, this.nuevoPath);
			} catch (HdtException e) {
				this.excep = e;
				return false;
			}
			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			OpcionesActivity.this.dialogSpinnerProgress.dismiss();
			if (!result) {
				DialogAlerta.mostrarError(this.excep);
			} else {
				OpcionesActivity.this.setCambioPreferenciaDirectorio(this.nuevoPath);
				OpcionesActivity.this.recreate();
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {

			OpcionesActivity.this.dialogSpinnerProgress = new ProgressDialog(OpcionesActivity.this);
			OpcionesActivity.this.dialogSpinnerProgress.setIndeterminate(false);
			OpcionesActivity.this.dialogSpinnerProgress.setCancelable(false);
			OpcionesActivity.this.dialogSpinnerProgress.setProgressStyle(ProgressDialog.STYLE_SPINNER);

			OpcionesActivity.this.dialogSpinnerProgress.setMessage(HdTIdioma.getString(R.string.progreso_moviendo_archivos));
			OpcionesActivity.this.dialogSpinnerProgress.setTitle(HdTIdioma.getString(R.string.progreso_moviendo_archivos_title));
			OpcionesActivity.this.dialogSpinnerProgress.show();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(java.lang.Object[])
		 */
		@Override
		protected void onProgressUpdate(Integer... values) {
		}
	}

	/** La notificacin del proceso. */
	private ProgressDialog dialogSpinnerProgress;

	/** The pref listener. */
	private SharedPreferences.OnSharedPreferenceChangeListener prefListener;

	/** The s prefs. */
	private SharedPreferences sPrefs;

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.preference.PreferenceActivity#onCreate(android.os.Bundle)
	 */
	@SuppressWarnings("deprecation")
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.addPreferencesFromResource(R.xml.preferences_hdtourist);
		this.setContentView(R.layout.activity_opciones);

		HdTouristService.getInstance(this);

		PreferenceManager.setDefaultValues(this, R.xml.preferences_hdtourist, true);
		this.setCambioPreferenciaDirectorio(PreferencesService.getDirectorio());

		this.checkPreferencesState(null);

		Preference filePicker = this.findPreference(PreferencesService.PREF_CONF_DIRECTORY);
		filePicker.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				OpcionesActivity.this.showFileChooser();
				return true;
			}
		});
		Preference porDefecto = this.findPreference("pref_por_defecto");
		porDefecto.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {

			@Override
			public boolean onPreferenceClick(Preference preference) {

				String directorioAnterior = PreferencesService.getDirectorio();
				SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(OpcionesActivity.this);
				Editor editor = sharedPrefs.edit();
				editor.clear();
				editor.commit();
				OpcionesActivity.this.procesarCambioDirectorio(directorioAnterior, FicherosService.obtenDirectorioPorDefecto());
				return true;
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#recreate()
	 */
	@Override
	public void recreate() {
		// if (Build.VERSION.SDK_INT < 11) {
		Intent intent = this.getIntent();
		this.overridePendingTransition(0, 0);
		intent.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
		this.finish();
		this.overridePendingTransition(0, 0);
		this.startActivity(intent);
		// } else {
		// super.recreate(); // En versiones nuevas parpadea, asi que dejamos el referesco de versiones viejas
		// }
	}

	/**
	 * Procesar cambio directorio.
	 *
	 * @param antiguoPath
	 *            el antiguo path
	 * @param path
	 *            el path
	 */
	private void procesarCambioDirectorio(final String antiguoPath, final String path) {
		File f = new File(path);

		if (antiguoPath.equals(path)) {
			this.recreate();
			return;
		}
		if (f.list() == null || f.list().length > 0) {
			boolean valido = true;
			for (String name : f.list()) {
				if (!Constantes.HDT_CARPETA.equals(name) && !Constantes.MAPAS_CARPETA.equals(name) && !Constantes.CONF_CARPETA.equals(name)
						&& !Constantes.CONF_IMAGENES.equals(name)) {
					valido = false;
				}
			}
			if (!valido) {
				DialogAlerta.mostrarAviso(this, HdTIdioma.getString(R.string.mensaje_aviso_directorio_title),
						HdTIdioma.getString(R.string.mensaje_aviso_directorio_configuracion)).show();

				return;
			}
		}

		new CambioDirectorioTask().execute(antiguoPath, path);

	}

	/**
	 * Fija el cambio preferencia directorio.
	 *
	 * @param path
	 *            el nuevo cambio preferencia directorio
	 */
	private void setCambioPreferenciaDirectorio(String path) {
		Preference conDirPreference = this.findPreference(PreferencesService.PREF_CONF_DIRECTORY);
		Editor editor = conDirPreference.getEditor();
		editor.putString(PreferencesService.PREF_CONF_DIRECTORY, path);
		editor.commit();
		conDirPreference.setSummary(path);
	}

	/**
	 * Show file chooser.
	 */
	private void showFileChooser() {
		Intent intent = new Intent(this, DirectoryChooserActivity.class);
		intent.putExtra(DirectoryChooserActivity.ONLY_DIRS, true);
		intent.putExtra(DirectoryChooserActivity.SHOW_HIDDEN, false);
		intent.putExtra(DirectoryChooserActivity.START_DIR, PreferencesService.getDirectorio());
		this.startActivityForResult(intent, DirectoryChooserActivity.PICK_DIRECTORY);
	}

	/**
	 * Aviso mapas en3 g.
	 *
	 * @return the dialog
	 */
	protected Dialog avisoMapasEn3G() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		builder.setMessage(
				"La descarga de mapas a travs de la red mvil puede consumir una gran cantidad de datos.\nEst seguro que desea activarla?")
				.setTitle(this.getResources().getString(R.string.preferencias_defecto_name_descargar_mapas_3g))
				.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int id) {
						dialog.cancel();
					}
				});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				CheckBoxPreference maps3GCheck = (CheckBoxPreference) OpcionesActivity.this.findPreference(PreferencesService.PREF_MAPAS_3G);
				maps3GCheck.setChecked(false);
				dialog.cancel();
			}
		});

		return builder.create();
	}

	/**
	 * Check preferences state.
	 *
	 * @param keyChanged
	 *            the key changed
	 */
	protected void checkPreferencesState(String keyChanged) {
		ListPreference conexionList = (ListPreference) this.findPreference(PreferencesService.PREF_CONEXION);
		EditTextPreference serverEditText = (EditTextPreference) this.findPreference(PreferencesService.PREF_SERVIDOR);
		CheckBoxPreference mapsGoogleCheck = (CheckBoxPreference) this.findPreference(PreferencesService.PREF_MAPAS_GOOGLE);
		CheckBoxPreference maps3GCheck = (CheckBoxPreference) this.findPreference(PreferencesService.PREF_MAPAS_3G);
		CheckBoxPreference imagenesCheck = (CheckBoxPreference) this.findPreference(PreferencesService.PREF_DESCARGAR_IMAGENES);

		if (maps3GCheck.isChecked()) {
			maps3GCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_mapas_3g_SI));
		} else {
			maps3GCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_mapas_3g_NO));
		}

		if (mapsGoogleCheck.isChecked()) {
			mapsGoogleCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_mapas_google_SI));
		} else {
			mapsGoogleCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_mapas_google_NO));
		}

		if (imagenesCheck.isChecked()) {
			imagenesCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_imagenes_si));
		} else {
			imagenesCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_imagenes_no));
		}

		if (PreferencesService.Conexion.NO == PreferencesService.getConexion()) {
			conexionList.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_conexion_NO));
			serverEditText.setEnabled(false);
			mapsGoogleCheck.setChecked(false);
			mapsGoogleCheck.setEnabled(false);
			maps3GCheck.setChecked(false);
			maps3GCheck.setEnabled(false);
			imagenesCheck.setEnabled(false);
			imagenesCheck.setChecked(false);
			maps3GCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_mapas_3g_NO_CON));
			mapsGoogleCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_mapas_google_NO));
			imagenesCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_imagenes_no));
		} else if (PreferencesService.Conexion.SI == PreferencesService.getConexion()) {
			conexionList.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_conexion_SI));
			serverEditText.setEnabled(true);
			mapsGoogleCheck.setEnabled(true);
			maps3GCheck.setEnabled(true);
			imagenesCheck.setEnabled(true);
			if (PreferencesService.PREF_CONEXION.equals(keyChanged)) {
				maps3GCheck.setChecked(false);
				maps3GCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_mapas_3g_NO));
			}
		} else if (PreferencesService.Conexion.PREGUNTAR == PreferencesService.getConexion()) {
			conexionList.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " "+ HdTIdioma.getString(R.string.preferencias_text_conexion_PREGUNTAR));
			serverEditText.setEnabled(true);
			mapsGoogleCheck.setEnabled(true);
			maps3GCheck.setEnabled(true);
			imagenesCheck.setEnabled(true);
			if (PreferencesService.PREF_CONEXION.equals(keyChanged)) {
				maps3GCheck.setChecked(false);
				maps3GCheck.setSummary(HdTIdioma.getString(R.string.preferencias_configurado_para) + " " + HdTIdioma.getString(R.string.preferencias_text_descargar_mapas_3g_NO));
			}
		}

		serverEditText.setSummary(serverEditText.getText());

		if (PreferencesService.PREF_MAPAS_3G.equals(keyChanged)) {
			if (maps3GCheck.isChecked()) {
				this.avisoMapasEn3G().show();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.preference.PreferenceActivity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == DirectoryChooserActivity.PICK_DIRECTORY) {
			if (data == null || data.getExtras() == null) {
				return;
			}
			Bundle extras = data.getExtras();
			String path = (String) extras.get(DirectoryChooserActivity.CHOSEN_DIRECTORY);
			if (path != null) {
				this.procesarCambioDirectorio(PreferencesService.getDirectorio(), path);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onResume()
	 */
	@Override
	protected void onResume() {
		super.onResume();
		this.sPrefs = PreferenceManager.getDefaultSharedPreferences(this);

		this.prefListener = new SharedPreferences.OnSharedPreferenceChangeListener() {
			@Override
			public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
				OpcionesActivity.this.checkPreferencesState(key);
			}
		};
		this.sPrefs.registerOnSharedPreferenceChangeListener(this.prefListener);
	}

}
