/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.base;

import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.utils.BarProgressDialog;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.IProgress;
import org.hdtourist.view.utils.NotificationProgress;
import org.hdtourist.view.utils.SpinnerProgressDialog;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.ActionBarActivity;

// TODO: Auto-generated Javadoc
/**
 * The Class BaseActionBarActivity.
 */
public abstract class BaseActionBarActivity extends ActionBarActivity implements IBaseVista {

	/** The errores handler. */
	private static Handler erroresHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			HdtException e = (HdtException) msg.obj;
			DialogAlerta.mostrarError(e);
		}
	};

	/** The dialog bar progress. */
	private IProgress dialogBarProgress;

	/** The dialog spinner progress. */
	private IProgress dialogSpinnerProgress;

	/** The notification progress. */
	private IProgress notificationProgress;

	/**
	 * Crea el notification progress.
	 */
	public void createNotificationProgress() {
		this.dialogSpinnerProgress = new SpinnerProgressDialog(this);
		this.dialogBarProgress = new BarProgressDialog(this);
		this.notificationProgress = new NotificationProgress(NotificationProgress.NOTIFICATION_MAPAS_ID, "Obteniedo mapas");
	}

	/**
	 * Gets the dialog bar progress.
	 *
	 * @return the dialog bar progress
	 */
	public IProgress getDialogBarProgress() {
		return this.dialogBarProgress;
	}

	/**
	 * Gets the dialog spiner progress.
	 *
	 * @return the dialog spiner progress
	 */
	public IProgress getDialogSpinerProgress() {
		return this.dialogSpinnerProgress;
	}

	/**
	 * Gets the notification progress.
	 *
	 * @return the notification progress
	 */
	public IProgress getNotificationProgress() {
		return this.notificationProgress;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.IBaseVista#manejaError(org.hdtourist.utils.HdtException)
	 */
	@Override
	public void manejaError(HdtException e) {
		Message msg = new Message();
		msg.what = 1;
		msg.obj = e;
		BaseActionBarActivity.erroresHandler.sendMessage(msg);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#recreate()
	 */
	@SuppressLint("NewApi")
	@Override
	public void recreate() {
		// if (Build.VERSION.SDK_INT < 11) {
		Intent intent = this.getIntent();
		this.overridePendingTransition(0, 0);
		intent.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
		this.finish();
		this.overridePendingTransition(0, 0);
		this.startActivity(intent);
		// } else { // En versiones nuevas parpadea, por lo que utilizamos el mtodo de versiones anteriores
		// super.recreate();
		// }
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v7.app.ActionBarActivity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		HdTouristService.getInstance(this);
		this.createNotificationProgress();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.FragmentActivity#onResume()
	 */
	@Override
	protected void onResume() {
		if (this.getDialogSpinerProgress().isShowing()) {
			this.getDialogSpinerProgress().setFinProceso();
		}
		if (this.getDialogBarProgress().isShowing()) {
			this.getDialogBarProgress().setFinProceso();
		}
		super.onResume();
	}
}