/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.inf;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hdtourist.R;
import org.hdtourist.model.Categoria;
import org.hdtourist.model.Consulta;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.InformacionFragment;
import org.hdtourist.view.inf.i.IInformacionObjetos;
import org.hdtourist.view.inf.i.IInformacionVisible;

import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

// TODO: Auto-generated Javadoc
/**
 * The Class InformacionDato.
 */
public class InformacionDato implements Comparable<InformacionDato>, IInformacionVisible, IInformacionObjetos {

	/** Si el dato es un vnculo, tendr una categora con la que rellenar la pantalla a la que lleva el vnculo. */
	private Categoria categoria;

	/** Consulta que informa el dato. */
	private Consulta consulta;

	/** Etiqueat que describe el dato. */
	private String etiqueta;

	/** The imagen text. */
	private boolean imagenText = false;

	/** The only text. */
	private boolean onlyText = false;

	/** Valores obtenidos de la consulta, A no ser que sea una lista, solo habr un valor. */
	private Set<String> valores;

	/** The max lines. */
	protected int MAX_LINES = 1;

	/**
	 * Instantiates a new informacion dato.
	 */
	public InformacionDato() {
		this.valores = new HashSet<String>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(InformacionDato another) {
		if (this.getValorVisible() != null) {
			return this.getValorVisible().compareTo(another.getValorVisible());
		} else {
			if (another.getValorVisible() != null) {
				return another.getValorVisible().compareTo(this.getValorVisible());
			} else {
				return 0;
			}
		}
	}

	/**
	 * Gets the categoria.
	 *
	 * @return the categoria
	 */
	public Categoria getCategoria() {
		return this.categoria;
	}

	/**
	 * Gets the consulta.
	 *
	 * @return the consulta
	 */
	public Consulta getConsulta() {
		return this.consulta;
	}

	/**
	 * Gets the etiqueta.
	 *
	 * @return the etiqueta
	 */
	public String getEtiqueta() {
		return this.etiqueta;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.i.IInformacionObjetos#getObjetosInformacion()
	 */
	@Override
	public List<IInformacionVisible> getObjetosInformacion() {
		return Arrays.asList((IInformacionVisible) this);
	}

	/**
	 * Gets the valores.
	 *
	 * @return the valores
	 */
	public Set<String> getValores() {
		return this.valores;
	}

	/**
	 * Gets the valor visible.
	 *
	 * @return the valor visible
	 */
	public String getValorVisible() {
		String valor = "";
		Iterator<String> i = this.getValores().iterator();
		if (i.hasNext()) {
			valor = i.next();
		}
		return valor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.i.IInformacionVisible#getView(org.hdtourist.view.InformacionFragment)
	 */
	@Override
	public View getView(final InformacionFragment actividad) {
		LayoutInflater inflater = actividad.getInflater();
		View view = inflater.inflate(R.layout.item_informacion, null, true);

		TextView etiquetaText = (TextView) view.findViewById(R.id.item_informacion_etiqueta);
		TextView valorText = (TextView) view.findViewById(R.id.item_informacion_valor);
		valorText.setMaxLines(this.MAX_LINES);
		etiquetaText.setText(this.getEtiqueta());
		valorText.setText(this.getValorVisible());

		if (this.isTooLarge(actividad, valorText, this.getValorVisible())) {
			this.fijaExpandirColapsar(actividad, view);
		}
		return view;
	}

	/**
	 * Checks if is imagen text.
	 *
	 * @return true, if is imagen text
	 */
	public boolean isImagenText() {
		return this.imagenText;
	}

	/**
	 * Checks if is only text.
	 *
	 * @return true, if is only text
	 */
	public boolean isOnlyText() {
		return this.onlyText;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.i.IInformacionVisible#isVisible(android.support.v4.app.Fragment)
	 */
	@Override
	public boolean isVisible(Fragment fragment) {
		// TODO Auto-generated method stub
		return StringUtils.isNotEmpty(this.getValorVisible());
	}

	/**
	 * Sets the categoria.
	 *
	 * @param categoria
	 *            the new categoria
	 */
	public void setCategoria(Categoria categoria) {
		this.categoria = categoria;
	}

	/**
	 * Sets the consulta.
	 *
	 * @param consulta
	 *            the new consulta
	 */
	public void setConsulta(Consulta consulta) {
		this.consulta = consulta;
	}

	/**
	 * Sets the etiqueta.
	 *
	 * @param etiqueta
	 *            the new etiqueta
	 */
	public void setEtiqueta(String etiqueta) {
		this.etiqueta = etiqueta;
	}

	/**
	 * Sets the imagen text.
	 *
	 * @param imagenText
	 *            the new imagen text
	 */
	public void setImagenText(boolean imagenText) {
		this.imagenText = imagenText;
	}

	/**
	 * Sets the only text.
	 *
	 * @param onlyText
	 *            the new only text
	 */
	public void setOnlyText(boolean onlyText) {
		this.onlyText = onlyText;
	}

	/**
	 * Sets the texto.
	 *
	 * @param texto
	 *            the new texto
	 */
	public void setTexto(String texto) {
		this.consulta.setTexto(texto);

	}

	/**
	 * Sets the valores.
	 *
	 * @param valores
	 *            the new valores
	 */
	public void setValores(Set<String> valores) {
		this.valores = valores;
	}

	/**
	 * Colocar imagen expandir.
	 *
	 * @param actividad
	 *            the actividad
	 * @param showImage
	 *            the show image
	 */
	private void colocarImagenExpandir(InformacionFragment actividad, ImageView showImage) {

		float anchoVentana = actividad.getActivity().getWindowManager().getDefaultDisplay().getWidth();

		RelativeLayout.LayoutParams newParams = new RelativeLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		newParams.setMargins((int) (anchoVentana - 100), 16 + 10 * this.MAX_LINES, 0, 0);
		showImage.setLayoutParams(newParams);
	}

	/**
	 * Fija expandir colapsar.
	 *
	 * @param actividad
	 *            the actividad
	 * @param view
	 *            the view
	 */
	protected void fijaExpandirColapsar(InformacionFragment actividad, View view) {
		final ImageView showImage = (ImageView) view.findViewById(R.id.show);
		final ImageView hideImage = (ImageView) view.findViewById(R.id.hide);
		final TextView valorAmplioText = (TextView) view.findViewById(R.id.valor_amplio);
		final TextView valorText = (TextView) view.findViewById(R.id.item_informacion_valor);
		final TextView etiquetaAmplioText = (TextView) view.findViewById(R.id.etiqueta_amplia);
		final RelativeLayout ampliadoView = (RelativeLayout) view.findViewById(R.id.ampliado);
		final RelativeLayout normalView = (RelativeLayout) view.findViewById(R.id.normal);

		etiquetaAmplioText.setText(this.getEtiqueta());
		valorAmplioText.setText(this.getValorVisible());
		if (this.onlyText) {
			normalView.setVisibility(View.GONE);
			ampliadoView.setVisibility(View.VISIBLE);
			showImage.setVisibility(View.GONE);
			hideImage.setVisibility(View.GONE);
			etiquetaAmplioText.setVisibility(View.GONE);
		} else {
			showImage.setVisibility(View.VISIBLE);

			this.colocarImagenExpandir(actividad, showImage);

			OnClickListener expand = new OnClickListener() {

				@Override
				public void onClick(View v) {
					normalView.setVisibility(View.GONE);
					ampliadoView.setVisibility(View.VISIBLE);
				}
			};

			OnClickListener collapse = new OnClickListener() {

				@Override
				public void onClick(View v) {
					normalView.setVisibility(View.VISIBLE);
					ampliadoView.setVisibility(View.GONE);
				}
			};

			showImage.setOnClickListener(expand);
			valorText.setOnClickListener(expand);

			hideImage.setOnClickListener(collapse);
			valorAmplioText.setOnClickListener(collapse);
		}
	}

	/**
	 * Checks if is too large.
	 *
	 * @param actividad
	 *            the actividad
	 * @param text
	 *            the text
	 * @param newText
	 *            the new text
	 * @return true, if is too large
	 */
	protected boolean isTooLarge(InformacionFragment actividad, TextView text, String newText) {
		float textWidth = text.getPaint().measureText(newText);

		float anchoVentana = actividad.getActivity().getWindowManager().getDefaultDisplay().getWidth();

		float anchoTextView = anchoVentana - text.getPaint().measureText(this.getEtiqueta()) - 8;
		return textWidth >= anchoTextView;
	}
}
