/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.inf;

import java.io.File;
import java.util.Iterator;

import org.hdtourist.R;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.InformacionFragment;
import org.hdtourist.view.inf.i.IInformacionVisible;
import org.hdtourist.view.utils.HdTIdioma;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.assist.ImageLoadingListener;

// TODO: Auto-generated Javadoc
/**
 * The Class InformacionImagen.
 */
public class InformacionImagen extends InformacionDato implements IInformacionVisible {

	/** The descripcion imagen. */
	private String descripcionImagen = "";

	/** The fragment. */
	private InformacionFragment fragment;

	/** The imagen. */
	private ImageView imagen;

	/** The texto. */
	private TextView texto;

	/** The image loading listener. */
	ImageLoadingListener imageLoadingListener = new ImageLoadingListener() {
		@Override
		public void onLoadingCancelled(String imageUri, View view) {
			InformacionImagen.this.imagen.setImageResource(R.drawable.ic_no_image);
			InformacionImagen.this.texto.setVisibility(View.GONE);
		}

		@Override
		public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
			InformacionImagen.this.setImage(loadedImage);
		}

		@Override
		public void onLoadingFailed(String arg0, View arg1, FailReason arg2) {
			InformacionImagen.this.imagen.setImageResource(R.drawable.ic_no_image);
			InformacionImagen.this.texto.setVisibility(View.GONE);

		}

		@Override
		public void onLoadingStarted(String imageUri, View view) {
			InformacionImagen.this.imagen.setImageResource(R.drawable.ic_image_loading);
			InformacionImagen.this.imagen.setEnabled(false);
			InformacionImagen.this.texto.setVisibility(View.GONE);
		}
	};

	/**
	 * Gets the descripcion imagen.
	 *
	 * @return the descripcion imagen
	 */
	public String getDescripcionImagen() {
		return this.descripcionImagen;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.InformacionDato#getView(org.hdtourist.view.InformacionFragment)
	 */
	@Override
	public View getView(final InformacionFragment fragment) {
		this.fragment = fragment;
		LayoutInflater inflater = fragment.getInflater();
		View view = inflater.inflate(R.layout.item_informacion_imagen, null, true);
		Iterator<String> i = this.getValores().iterator();

		String aux = "";
		if (i.hasNext()) {
			aux = i.next();
		}
		final String imagenUrl = aux;

		this.imagen = (ImageView) view.findViewById(R.id.item_informacion_imagen);
		this.texto = (TextView) view.findViewById(R.id.item_informacion_imagen_texto);
		// Si la imagen est cacheada la muestro de la cache
		File cachedImage = ImageLoader.getInstance().getDiscCache().get(imagenUrl);
		if (cachedImage.exists()) {
			this.texto.setVisibility(View.GONE);
			DisplayImageOptions options = new DisplayImageOptions.Builder().cacheOnDisc().build();

			ImageLoader.getInstance().loadImage(imagenUrl, options, this.imageLoadingListener);
		} else if (PreferencesService.getDescargarImagenes() && PreferencesService.isConexionSiempre()) {
			this.texto.setVisibility(View.GONE);
			ImageLoader.getInstance().loadImage(imagenUrl, this.imageLoadingListener);
		} else if (PreferencesService.getDescargarImagenes() && PreferencesService.isConexionPreguntar()) {
			this.texto.setVisibility(View.VISIBLE);
			this.texto.setText(R.string.informacion_imagen_click_descargar);
			View.OnClickListener list = new View.OnClickListener() {

				@Override
				public void onClick(View v) {

					AlertDialog.Builder builder = new AlertDialog.Builder(fragment.getActivity());
					builder.setMessage(HdTIdioma.getString(R.string.informacion_imagen_aviso))
					.setTitle(HdTIdioma.getString(R.string.informacion_imagen_aviso_title))
					.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							dialog.dismiss();
							ImageLoader.getInstance().loadImage(imagenUrl, InformacionImagen.this.imageLoadingListener);
						}
					});
					builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.cancel();
						}
					});

					builder.create().show();
				}
			};

			this.imagen.setOnClickListener(list);
			this.texto.setOnClickListener(list);
		}
		return view;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.InformacionDato#isVisible(android.support.v4.app.Fragment)
	 */
	@Override
	public boolean isVisible(Fragment fragment) {
		if (StringUtils.isEmpty(this.getValorVisible())) {
			return false;
		}
		String imagenUrl = this.getValorVisible();
		File cachedImage = ImageLoader.getInstance().getDiscCache().get(imagenUrl);

		return PreferencesService.getDescargarImagenes() || cachedImage.exists();
	}

	/**
	 * Sets the descripcion imagen.
	 *
	 * @param descripcionImagen
	 *            the new descripcion imagen
	 */
	public void setDescripcionImagen(String descripcionImagen) {
		this.descripcionImagen = descripcionImagen;
	}

	/**
	 * Sets the image.
	 *
	 * @param result
	 *            the new image
	 */
	private void setImage(Bitmap result) {
		try {
			this.imagen.setImageBitmap(result);
			this.imagen.setScaleType(ImageView.ScaleType.CENTER_INSIDE);
			LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) this.imagen.getLayoutParams();
			int width = this.fragment.getActivity().getWindowManager().getDefaultDisplay().getWidth();
			int newHeight = width * result.getHeight() / result.getWidth();
			params.height = newHeight;
			this.imagen.setLayoutParams(params);

			if (StringUtils.isNotEmpty(this.descripcionImagen)) {
				this.texto.setText(this.descripcionImagen);
				this.texto.setVisibility(View.VISIBLE);
			} else {
				this.texto.setVisibility(View.GONE);
			}
		} catch (Exception e) {

		}
	}
}
