/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.inf;

import java.util.Iterator;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.GeoPosicion;
import org.hdtourist.model.ListaGeoPosiciones;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.InformacionService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.presenter.InformacionPresenter;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.InformacionFragment;
import org.hdtourist.view.MapaActivity;
import org.hdtourist.view.inf.i.IInformacionVisible;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * La clase InformacionMapa.
 */
public class InformacionMapa extends InformacionDato implements IInformacionVisible {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.InformacionDato#getView(org.hdtourist.view.InformacionFragment)
	 */
	@Override
	public View getView(final InformacionFragment actividad) {

		LayoutInflater inflater = actividad.getInflater();
		View view = inflater.inflate(R.layout.item_informacion_mapa, null, true);

		Iterator<String> i = this.getValores().iterator();

		TextView textViewEt = (TextView) view.findViewById(R.id.item_informacion_mapa_etiqueta);
		ImageView boton = (ImageView) view.findViewById(R.id.item_informacion_mapa_imagen);
		textViewEt.setText(this.getEtiqueta());
		String coordenadas = "";
		if (i.hasNext()) {
			coordenadas = i.next();
		}
		boton.setTag(R.integer.coordenadas, coordenadas);
		boton.setTag(R.integer.titulo, InformacionPresenter.getInstance(actividad).getCategoria().getNombre());
		boton.setTag(R.integer.ciudad, InformacionPresenter.getInstance(actividad).getCiudad());
		boton.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View view) {
				String coordenadas = (String) view.getTag(R.integer.coordenadas);
				final String etiqueta = (String) view.getTag(R.integer.titulo);
				final FicheroHdt ciudad = (FicheroHdt) view.getTag(R.integer.ciudad);
				try {
					final GeoPosicion geo = InformacionService.getInstance().corregirPunto(InformacionPresenter.getInstance(actividad).getCiudad(),
							GeoPosicion.getGeoPosicion(coordenadas));
					if (PreferencesService.getUsarMapasGoogle()) {
						if (PreferencesService.isConexionNunca()) {
							DialogAlerta.mostrarAvisoPreferencias(actividad.getActivity(),
									HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada_title),
									HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada)).show();
							return;
						}
						if (!HdTouristService.getInstance().isNetworkAvailable()) {
							if (ciudad.hayMapasOffline()) {
								AlertDialog.Builder builder = new AlertDialog.Builder(actividad.getActivity());

								builder.setMessage(HdTIdioma.getString(R.string.informacion_mapas_error_no_conexion))
								.setTitle(HdTIdioma.getString(R.string.mensaje_error_no_conexion_title))
								.setPositiveButton(HdTIdioma.getString(R.string.dialog_mapa_offline_button),
										new DialogInterface.OnClickListener() {
									@Override
									public void onClick(DialogInterface dialog, int id) {
										abrirMapasOffline(actividad, etiqueta, ciudad, geo);
									}
								});
								builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_cancelar_button),
										new DialogInterface.OnClickListener() {
									@Override
									public void onClick(DialogInterface dialog, int which) {
										dialog.cancel();
									}
								});
								builder.create().show();
								return;
							} else {
								DialogAlerta.mostrarAviso(actividad.getActivity(), HdTIdioma.getString(R.string.mensaje_error_no_conexion_title),
										HdTIdioma.getString(R.string.informacion_mapas_error_no_conexion_title)).show();
								return;
							}
						}
						this.abrirGoogleMapsConfirmando(actividad, etiqueta, geo);
					} else {
						this.abrirMapasOffline(actividad, etiqueta, ciudad, geo);
					}

				} catch (Exception e) {
					DialogAlerta.mostrarError(new HdtException(HdTIdioma.getString(R.string.mensaje_error_mostrar_mapa)));
				}

			}

			private void abrirGoogleMaps(final InformacionFragment actividad, String etiqueta, GeoPosicion geo) {
				Intent intent;
				String uriBegin = "geo:" + geo.latitude + "," + geo.longitude;
				String query = geo.latitude + "," + geo.longitude + "(" + etiqueta + ")";
				String encodedQuery = Uri.encode(query);
				String uriString = uriBegin + "?q=" + encodedQuery + "&z=16";
				Uri uri = Uri.parse(uriString);
				intent = new Intent(android.content.Intent.ACTION_VIEW, uri);
				actividad.startActivityForResult(intent, 0000);
			}

			private void abrirGoogleMapsConfirmando(final InformacionFragment actividad, final String etiqueta, final GeoPosicion geo) {
				if (PreferencesService.isConexionPreguntar()) {
					final String etiquetaAux = etiqueta;
					final GeoPosicion geoAux = geo;
					AlertDialog.Builder builder = new AlertDialog.Builder(actividad.getActivity());

					builder.setMessage(HdTIdioma.getString(R.string.informacion_mapas_aviso_conexion))
					.setTitle(HdTIdioma.getString(R.string.informacion_mapas_aviso_conexion_title))
					.setPositiveButton(HdTIdioma.getString(R.string.dialog_mapa_button), new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							abrirGoogleMaps(actividad, etiquetaAux, geoAux);
						}
					});
					builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_cancelar_button), new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.cancel();
						}
					});
					builder.create().show();
				} else {
					this.abrirGoogleMaps(actividad, etiqueta, geo);
				}
			}

			private void abrirMapasOffline(final InformacionFragment fragment, String etiqueta, FicheroHdt ciudad, GeoPosicion geo) {
				if (!ciudad.hayMapasOffline()) {
					DialogAlerta.mostrarAviso(fragment.getActivity(), HdTIdioma.getString(R.string.informacion_mapas_offline_no_disponible_title),
							HdTIdioma.getString(R.string.informacion_mapas_offline_no_disponible)).show();
					return;
				}
				Intent intent = new Intent(fragment.getActivity(), MapaActivity.class);

				ListaGeoPosiciones lista = new ListaGeoPosiciones();
				lista.setFichero(ciudad);
				lista.add(geo, etiqueta);

				intent.putExtra("LISTA", lista);
				fragment.startActivityForResult(intent, 0000);
			}
		});

		return view;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.inf.InformacionDato#isVisible(android.support.v4.app.Fragment)
	 */
	@Override
	public boolean isVisible(Fragment fragment) {
		// TODO Auto-generated method stub
		return InformacionPresenter.getInstance((InformacionFragment) fragment).mostrarSituacion();
	}

}
