/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.utils;

import org.hdtourist.R;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.OpcionesActivity;
import org.hdtourist.view.base.BaseActionBarActivity;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;

/**
 * The Class DialogAlerta.
 */
public class DialogAlerta {

	/**
	 * Mostrar aviso.
	 *
	 * @param contexto
	 *            the contexto
	 * @param titulo
	 *            the titulo
	 * @param mensaje
	 *            the mensaje
	 * @return the dialog
	 */
	public static Dialog mostrarAviso(Context contexto, String titulo, String mensaje) {

		AlertDialog.Builder builder = new AlertDialog.Builder(contexto);

		builder.setMessage(mensaje).setTitle(titulo)
		.setPositiveButton(HdTIdioma.getString(R.string.dialog_ok_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				dialog.cancel();
			}
		});

		return builder.create();
	}

	/**
	 * Mostrar aviso preferencias.
	 *
	 * @param contexto
	 *            the contexto
	 * @param titulo
	 *            the titulo
	 * @param mensaje
	 *            the mensaje
	 * @return the dialog
	 */
	public static Dialog mostrarAvisoPreferencias(final Context contexto, String titulo, String mensaje) {

		AlertDialog.Builder builder = new AlertDialog.Builder(contexto);

		builder.setMessage(mensaje).setTitle(titulo)
		.setPositiveButton(HdTIdioma.getString(R.string.dialog_ir_preferencias_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				Intent intent = new Intent(contexto, OpcionesActivity.class);
				contexto.startActivity(intent);
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_cancelar_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});

		return builder.create();
	}

	/**
	 * Mostrar confirmacion pantalla.
	 *
	 * @param contexto
	 *            the contexto
	 * @param titulo
	 *            the titulo
	 * @param mensaje
	 *            the mensaje
	 * @param intent
	 *            the intent
	 * @return the dialog
	 */
	public static Dialog mostrarConfirmacionPantalla(final Context contexto, String titulo, String mensaje, final Intent intent) {

		AlertDialog.Builder builder = new AlertDialog.Builder(contexto);

		builder.setMessage(mensaje).setTitle(titulo)
		.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				dialog.dismiss();
				((BaseActionBarActivity) contexto).getDialogSpinerProgress().setInicioProceso();

				((BaseActionBarActivity) contexto).getDialogSpinerProgress().setInicioProceso();
				((BaseActionBarActivity) contexto).getDialogSpinerProgress().setTitulo("");
				((BaseActionBarActivity) contexto).getDialogSpinerProgress().setTexto(
						HdTIdioma.getString(R.string.progreso_obteniendo_ciudades_servidor));
				contexto.startActivity(intent);
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});

		return builder.create();
	}

	/**
	 * Mostrar error.
	 *
	 * @param e
	 *            the e
	 */
	public static void mostrarError(HdtException e) {

		AlertDialog.Builder builder = new AlertDialog.Builder(HdTouristService.getInstance().getContexto());
		String mensaje = HdTIdioma.getString(R.string.mensaje_error_ha_ocurrido_title);
		if (e != null) {
			mensaje = e.getMensaje();
		}
		builder.setMessage(mensaje).setTitle(HdTIdioma.getString(R.string.mensaje_error_ha_ocurrido_title))
		.setPositiveButton(HdTIdioma.getString(R.string.dialog_ok_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				dialog.cancel();
			}
		});

		e.printStackTrace();

		builder.create().show();
	}

}
