/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.utils;

import java.io.File;
import java.util.List;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.InformacionService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.presenter.CiudadesDisponiblesPresenter;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.CiudadesDisponiblesListFragment;
import org.hdtourist.view.InfoHDTActivity;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.support.v7.widget.PopupMenu;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.assist.ImageLoadingListener;

// TODO: Auto-generated Javadoc
/**
 * The Class ListaCiudadesDisponiblesAdapter.
 */
public class ListaCiudadesDisponiblesAdapter extends BaseAdapter {

	/** The fragment. */
	private final CiudadesDisponiblesListFragment fragment;

	/** The lista. */
	private final List<FicheroHdt> lista;

	/** The presenter. */
	private CiudadesDisponiblesPresenter presenter;

	/** The click listener mapa. */
	View.OnClickListener clickListenerMapa = new View.OnClickListener() {

		@Override
		public void onClick(View view) {
			FicheroHdt elegido = (FicheroHdt) view.getTag();
			if (!elegido.hayMapasOffline()) {
				// Comprobamos las preferencias para saber si tiene que preguntar antes de conectarse a internet.
				ListaCiudadesDisponiblesAdapter.this.confirmarYDescargarMapa(elegido);

			} else {
				ListaCiudadesDisponiblesAdapter.this.presenter.showMap(elegido);
			}
		}
	};

	/** The click listener menu. */
	View.OnClickListener clickListenerMenu = new View.OnClickListener() {

		@Override
		public void onClick(View view) {
			ListaCiudadesDisponiblesAdapter.this.showPopup(view);
		}
	};

	/** The click listener seleccion. */
	View.OnClickListener clickListenerSeleccion = new View.OnClickListener() {

		@Override
		public void onClick(View view) {
			FicheroHdt elegido = (FicheroHdt) view.getTag();

			File fc = new File(elegido.getRutaFichero());
			if (!fc.exists()) {

				ListaCiudadesDisponiblesAdapter.this.confirmarYDescargarHdt(elegido);
				// Comprobamos las preferencias para saber si tiene que preguntar antes de conectarse a internet.

			} else {
				ListaCiudadesDisponiblesAdapter.this.presenter.openCiudadActivity(elegido);
			}
		}
	};

	/** The image loading listener. */
	ImageLoadingListener imageLoadingListener = new ImageLoadingListener() {
		@Override
		public void onLoadingCancelled(String imageUri, View view) {
			if (view != null) {
				((ImageView) view).setImageResource(R.drawable.ic_city);
			}
		}

		@Override
		public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
			try {
				((ImageView) view).setImageBitmap(loadedImage);
				((ImageView) view).setScaleType(ImageView.ScaleType.CENTER_INSIDE);
				LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) ((ImageView) view).getLayoutParams();
				int width = ListaCiudadesDisponiblesAdapter.this.fragment.getActivity().getWindowManager().getDefaultDisplay().getWidth() / 4;
				int newHeight = width * loadedImage.getHeight() / loadedImage.getWidth();
				if (newHeight > 2 * width) {
					newHeight = 2 * width;
				}
				params.height = newHeight;
				params.width = width;
				((ImageView) view).setLayoutParams(params);
			} catch (Exception e) {

			}
		}

		@Override
		public void onLoadingFailed(String arg0, View view, FailReason arg2) {
			if (view != null) {
				((ImageView) view).setImageResource(R.drawable.ic_city);
			}

		}

		@Override
		public void onLoadingStarted(String imageUri, View view) {
			((ImageView) view).setImageResource(R.drawable.ic_city);
			view.setEnabled(false);
		}
	};

	/**
	 * Instantiates a new lista ciudades disponibles adapter.
	 *
	 * @param presenter
	 *            the presenter
	 * @param lista
	 *            the lista
	 * @param ciudadesDisponiblesListFragment
	 *            the ciudades disponibles list fragment
	 */
	public ListaCiudadesDisponiblesAdapter(CiudadesDisponiblesPresenter presenter, List<FicheroHdt> lista,
			CiudadesDisponiblesListFragment ciudadesDisponiblesListFragment) {
		this.lista = lista;
		this.fragment = ciudadesDisponiblesListFragment;
		this.presenter = presenter;
	}

	/**
	 * Confirmar y descargar hdt.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void confirmarYDescargarHdt(final FicheroHdt elegido) {
		if (PreferencesService.isConexionSiempre()) {
			this.descargarCiudad(elegido);
			return;
		}

		if (PreferencesService.isConexionNunca()) {
			DialogAlerta.mostrarAvisoPreferencias(this.fragment.getActivity(), HdTIdioma.getString(R.string.mensaje_error_ha_ocurrido_title),
					HdTIdioma.getString(R.string.mensaje_error_archivo_activar)).show();
			return;
		}
		AlertDialog.Builder builder = new AlertDialog.Builder(this.fragment.getActivity());

		builder.setTitle(HdTIdioma.getString(R.string.mensaje_aviso_archivo_title));
		builder.setMessage(HdTIdioma.getString(R.string.mensaje_aviso_archivo));
		builder.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {

				ListaCiudadesDisponiblesAdapter.this.descargarCiudad(elegido);
				dialog.cancel();
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				String message = HdTIdioma.getString(R.string.mensaje_aviso_archivo_seleccione_otra);
				Toast.makeText(ListaCiudadesDisponiblesAdapter.this.fragment.getActivity(), message, Toast.LENGTH_LONG).show();
			}
		});

		builder.create().show();
	}

	/**
	 * Confirmar y descargar mapa.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void confirmarYDescargarMapa(final FicheroHdt elegido) {
		if (PreferencesService.isConexionNunca()) {
			DialogAlerta.mostrarAvisoPreferencias(this.fragment.getActivity(),
					HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada_title),
					HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada)).show();
			return;
		}
		// SE COMENTA PARA QUE LA DESCARGA DE MAPAS SIEMPRE SE CONSULTE. AL SER ARCHIVOS GRANDES PUEDE CONSUMIR DEMASIADOS DATOS AL USUARIO.
		// if (PreferencesService.isConexionSiempre()) {
		// this.descargarMapa(elegido);
		// return;
		// }
		AlertDialog.Builder builder = new AlertDialog.Builder(this.fragment.getActivity());

		builder.setTitle(HdTIdioma.getString(R.string.mensaje_aviso_mapas_descargar_title));
		builder.setMessage(HdTIdioma.getString(R.string.mensaje_aviso_mapas_descargar, FicherosService.getStringSize(elegido.getTamanoMaps())));
		builder.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				ListaCiudadesDisponiblesAdapter.this.descargarMapa(elegido);
				dialog.cancel();
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
			}
		});

		builder.create().show();
	}

	/**
	 * Descargar ciudad.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void descargarCiudad(FicheroHdt elegido) {
		Log.i("Descarga", "Descargando ciudad.");
		if (!HdTouristService.getInstance().isNetworkAvailable()) {
			DialogAlerta.mostrarAviso(this.fragment.getActivity(), HdTIdioma.getString(R.string.mensaje_error_no_conexion_title),
					HdTIdioma.getString(R.string.mensaje_error_no_conexion)).show();
			return;
		}
		this.presenter.descargaCiudad(elegido);
	}

	/**
	 * Descargar mapa.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void descargarMapa(FicheroHdt elegido) {
		Log.i("Descarga", "Descargando MAPA.");
		if (!HdTouristService.getInstance().isNetworkAvailable()) {
			DialogAlerta.mostrarAviso(this.fragment.getActivity(), HdTIdioma.getString(R.string.mensaje_error_no_conexion_title),
					HdTIdioma.getString(R.string.mensaje_error_no_conexion)).show();
			return;
		}
		ConnectivityManager connManager = (ConnectivityManager) this.fragment.getActivity().getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo mWifi = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);

		if (mWifi.isConnected()) {
			this.presenter.descargarMapa(elegido);
		} else if (!PreferencesService.getDescargarMapas3G()) {
			DialogAlerta.mostrarAvisoPreferencias(this.fragment.getActivity(), HdTIdioma.getString(R.string.informacion_mapas_wifi_title),
					HdTIdioma.getString(R.string.informacion_mapas_wifi)).show();
		} else {
			this.presenter.descargarMapa(elegido);
		}
	}

	/**
	 * Eliminar ciudad.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void eliminarCiudad(FicheroHdt elegido) {
		this.presenter.eliminaCiudad(elegido);
	}

	/**
	 * Eliminar mapa.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void eliminarMapa(FicheroHdt elegido) {
		Log.i("Descarga", "Descargando ciudad.");
		this.presenter.eliminaMapa(elegido);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getCount()
	 */
	@Override
	public int getCount() {
		return this.lista.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	@Override
	public Object getItem(int position) {
		return this.lista.get(position);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItemId(int)
	 */
	@Override
	public long getItemId(int position) {
		return position;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
	@Override
	public View getView(int position, View arg1, ViewGroup arg2) {
		LayoutInflater inflater = this.fragment.getActivity().getLayoutInflater();
		View view = inflater.inflate(R.layout.item_ciudad, null, true);
		TextView textView = (TextView) view.findViewById(R.id.ciudadNombre);
		FicheroHdt fichero = this.lista.get(position);
		textView.setText(fichero.getNombre());

		ImageView myImage = (ImageView) view.findViewById(R.id.ciudadImagen);
		myImage.setOnClickListener(this.clickListenerSeleccion);
		myImage.setTag(fichero);
		try {
			final String imagenUrl = InformacionService.getInstance().obtenerUrlImagenCiudad(fichero);

			// Si la imagen est cacheada la muestro de la cache
			File cachedImage = ImageLoader.getInstance().getDiscCache().get(imagenUrl);

			if (cachedImage.exists()) {
				DisplayImageOptions options = new DisplayImageOptions.Builder().cacheOnDisc().build();
				ImageLoader.getInstance().displayImage(imagenUrl, myImage, options, this.imageLoadingListener);
			} else if (PreferencesService.getDescargarImagenes() && PreferencesService.isConexionSiempre()) {
				ImageLoader.getInstance().displayImage(imagenUrl, myImage, this.imageLoadingListener);
			} else if (PreferencesService.getDescargarImagenes() && PreferencesService.isConexionPreguntar()) {
				final ImageView im = myImage;
				View.OnClickListener list = new View.OnClickListener() {
					ImageView myIm = im;

					@Override
					public void onClick(View v) {

						AlertDialog.Builder builder = new AlertDialog.Builder(ListaCiudadesDisponiblesAdapter.this.fragment.getActivity());
						builder.setMessage(HdTIdioma.getString(R.string.informacion_imagen_aviso))
								.setTitle(HdTIdioma.getString(R.string.informacion_imagen_aviso_title))
								.setPositiveButton(HdTIdioma.getString(R.string.dialog_si_button), new DialogInterface.OnClickListener() {
									@Override
									public void onClick(DialogInterface dialog, int id) {
										dialog.dismiss();
										ImageLoader.getInstance().displayImage(imagenUrl, myIm,
												ListaCiudadesDisponiblesAdapter.this.imageLoadingListener);
										myIm.setOnClickListener(ListaCiudadesDisponiblesAdapter.this.clickListenerSeleccion);
									}
								});
						builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_no_button), new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog, int which) {
								dialog.cancel();
							}
						});

						builder.create().show();
					}
				};

				myImage.setOnClickListener(list);
			}

		} catch (HdtException e) {
		}

		textView.setOnClickListener(this.clickListenerSeleccion);
		textView.setTag(fichero);

		ImageView menuImage = (ImageView) view.findViewById(R.id.ciudad_menu);
		menuImage.setOnClickListener(this.clickListenerMenu);
		menuImage.setTag(fichero);

		ImageView myImageMap = (ImageView) view.findViewById(R.id.ciudadMapa);
		if (fichero.hayMapasOffline()) {
			myImageMap.setImageResource(R.drawable.ic_action_map_si);
		} else {
			myImageMap.setImageResource(R.drawable.ic_action_map_no);
		}
		myImageMap.setOnClickListener(this.clickListenerMapa);
		myImageMap.setTag(fichero);
		return view;
	}

	/**
	 * Show popup.
	 *
	 * @param itemButton
	 *            the item button
	 */
	public void showPopup(View itemButton) {
		ImagePopupMenu popup = new ImagePopupMenu(this.fragment.getActivity(), itemButton);
		popup.setSubMenu(R.menu.menu_item_ciudad);
		final FicheroHdt fichero = (FicheroHdt) itemButton.getTag();
		if (fichero.hayMapasOffline()) {
			popup.getMenuItem(R.id.menu_descargar_mapa).setVisible(false);
			popup.getMenuItem(R.id.menu_eliminar_mapa).setVisible(true);
		} else {
			popup.getMenuItem(R.id.menu_descargar_mapa).setVisible(true);
			popup.getMenuItem(R.id.menu_eliminar_mapa).setVisible(false);
		}
		popup.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {

			@Override
			public boolean onMenuItemClick(MenuItem item) {
				switch (item.getItemId()) {
				case R.id.menu_descargar_mapa:
					ListaCiudadesDisponiblesAdapter.this.confirmarYDescargarMapa(fichero);
					return true;
				case R.id.menu_eliminar_mapa:
					ListaCiudadesDisponiblesAdapter.this.eliminarMapa(fichero);
					return true;
				case R.id.menu_eliminar_ciudad:
					ListaCiudadesDisponiblesAdapter.this.eliminarCiudad(fichero);
					return true;
				case R.id.menu_informacion_hdt:
					ListaCiudadesDisponiblesAdapter.this.informacionHDT(fichero);
					return true;
				}
				return false;

			}
		});

		popup.show();

	}

	protected void informacionHDT(FicheroHdt fichero) {
		Intent intent = new Intent(this.fragment.getActivity(), InfoHDTActivity.class);
		intent.putExtra("HDT", fichero);
		this.fragment.getActivity().startActivityForResult(intent, 0000);

	}
}
