/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.utils;

import java.util.ArrayList;
import java.util.List;

import android.location.Location;

import com.nutiteq.components.Color;
import com.nutiteq.components.MapPos;
import com.nutiteq.geometry.Line;
import com.nutiteq.projections.Projection;
import com.nutiteq.style.LineStyle;
import com.nutiteq.utils.Const;
import com.nutiteq.vectorlayers.GeometryLayer;

// TODO: Auto-generated Javadoc
/**
 * The Class LocalizacionView.
 */
public class LocalizacionView {

	/** La constante NR_OF_CIRCLE_VERTS. */
	private static final int NR_OF_CIRCLE_VERTS = 18;

	/** The circle. */
	private Line circle = null;

	/** The circle pos. */
	private MapPos circlePos = new MapPos(0, 0);

	/** The circle radius. */
	private float circleRadius = 1;

	/** The circle scale. */
	private float circleScale = 0;

	/** The circle verts. */
	private List<MapPos> circleVerts = new ArrayList<MapPos>(LocalizacionView.NR_OF_CIRCLE_VERTS);

	/** The layer. */
	private final GeometryLayer layer;

	/** The projection scale. */
	private float projectionScale = 0;

	/** The visible. */
	private boolean visible = false;

	/**
	 * Instantiates a new localizacion view.
	 *
	 * @param layer
	 *            the layer
	 */
	public LocalizacionView(GeometryLayer layer) {
		this.layer = layer;
	}

	/**
	 * Fija el location.
	 *
	 * @param proj
	 *            the proj
	 * @param location
	 *            the location
	 */
	public void setLocation(Projection proj, Location location) {
		this.circlePos = proj.fromWgs84(location.getLongitude(), location.getLatitude());
		this.projectionScale = (float) proj.getBounds().getWidth();
		this.circleRadius = location.getAccuracy();
	}

	/**
	 * Sets the visible.
	 *
	 * @param visible
	 *            the new visible
	 */
	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	/**
	 * Update.
	 *
	 * @param zoom
	 *            the zoom
	 */
	public void update(float zoom) {
		// circle max radius
		// make sure that it is at least minimum radius, otherwise is too small in general zoom
		float zoomPow2 = (float) Math.pow(2, zoom);
		float circleScaleMax = Math.max(this.circleRadius / 7500000f, // based on GPS accuracy. This constant depends on latitude
				1.0f / zoomPow2 * 0.2f) * this.projectionScale; // minimum, fixed value
		float circleScaleStep = circleScaleMax / 50.0f;

		this.circleScale += circleScaleStep;
		if (this.circleScale > circleScaleMax) {
			this.circleScale = 0.0f;
		}

		// Build closed circle
		this.circleVerts.clear();
		for (float tsj = 0; tsj <= 360; tsj += 360 / LocalizacionView.NR_OF_CIRCLE_VERTS) {
			MapPos mapPos = new MapPos(this.circleScale * Math.cos(tsj * Const.DEG_TO_RAD) + this.circlePos.x, this.circleScale
					* Math.sin(tsj * Const.DEG_TO_RAD) + this.circlePos.y);
			this.circleVerts.add(mapPos);
		}

		// Create/update line
		if (this.circle == null) {
			LineStyle style = LineStyle.builder().setWidth(0.1f).setColor(Color.BLUE).build();
			this.circle = new Line(this.circleVerts, null, style, null);
			this.layer.add(this.circle);
		} else {
			this.circle.setVertexList(this.circleVerts);
		}
		this.circle.setVisible(this.visible);
	}
}
