/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.utils;

import org.hdtourist.R;
import org.hdtourist.model.services.HdTouristService;

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.support.v4.app.NotificationCompat;

// TODO: Auto-generated Javadoc
/**
 * The Class NotificationProgress.
 */
public class NotificationProgress extends IProgress {

	/** La constante NOTIFICATION_MAPAS_ID. */
	public static final int NOTIFICATION_MAPAS_ID = 342534;

	/** La constante NOTIFICATION_SEG. */
	public static final int NOTIFICATION_SEG = 5;

	/** The contexto. */
	private Context contexto;

	/** The id. */
	private int ID;

	/** The m builder. */
	private NotificationCompat.Builder mBuilder;

	/** The m notify manager. */
	private NotificationManager mNotifyManager;

	/** The notificacion anterior. */
	private long notificacionAnterior = 0;

	/** The num procesos. */
	private int numProcesos = 1;

	/** The num procesos pendientes. */
	private int numProcesosPendientes = 1;

	/** The texto del progreso. */
	private String textoDelProgreso = "";

	/**
	 * Instantiates a new notification progress.
	 *
	 * @param id
	 *            the id
	 * @param title
	 *            the title
	 */
	public NotificationProgress(int id, String title) {
		this.contexto = HdTouristService.getInstance().getContexto();
		this.ID = id;

		PendingIntent resultPendingIntent = PendingIntent.getActivity(this.contexto, 0, new Intent(), PendingIntent.FLAG_UPDATE_CURRENT);

		this.mNotifyManager = (NotificationManager) this.contexto.getSystemService(Context.NOTIFICATION_SERVICE);
		this.mBuilder = new NotificationCompat.Builder(this.contexto.getApplicationContext()).setSmallIcon(R.drawable.ic_city).setContentTitle(title)
				.setContentText("Contexto").setContentIntent(resultPendingIntent).setAutoCancel(false);
	}

	/**
	 * Instantiates a new notification progress.
	 *
	 * @param intent
	 *            the intent
	 * @param id
	 *            the id
	 * @param title
	 *            the title
	 */
	public NotificationProgress(Intent intent, int id, String title) {
		this.contexto = HdTouristService.getInstance().getContexto();
		this.ID = id;

		PendingIntent resultPendingIntent = PendingIntent.getActivity(this.contexto, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);

		this.mNotifyManager = (NotificationManager) this.contexto.getSystemService(Context.NOTIFICATION_SERVICE);
		this.mBuilder = new NotificationCompat.Builder(this.contexto.getApplicationContext()).setSmallIcon(R.drawable.ic_city).setContentTitle(title)
				.setContentText("Contexto").setContentIntent(resultPendingIntent).setAutoCancel(false);
	}

	/**
	 * Fuerza la notificacin, no hacer muchas llamadas seguidas a este mtodo ya que se puede bloquear el terminal.
	 */
	@Override
	public void forzarNotificacion() {
		this.mNotifyManager.notify(this.ID, this.mBuilder.build());
		this.notificacionAnterior = System.currentTimeMillis();
	}

	/**
	 * Gets the num procesos.
	 *
	 * @return the numProcesos
	 */
	@Override
	public int getNumProcesos() {
		return this.numProcesos;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#isShowing()
	 */
	@Override
	public boolean isShowing() {
		return true;
	}

	/**
	 * Actualiza la notificacin, solo si han pasado NOTIFICACION_SEG segundos (para que nose bloquee).
	 */
	@Override
	public void notifica() {
		if (System.currentTimeMillis() > this.notificacionAnterior + NotificationProgress.NOTIFICATION_SEG * 1000) {
			this.mNotifyManager.notify(this.ID, this.mBuilder.build());
			this.notificacionAnterior = System.currentTimeMillis();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setErrorProceso()
	 */
	@Override
	public void setErrorProceso() {
		this.setTexto("Ocurrio un error");
		this.mBuilder.setAutoCancel(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setFinProceso()
	 */
	@Override
	public void setFinProceso() {
		this.numProcesosPendientes--;
		this.mBuilder.setProgress(100, 100, false);
		this.mBuilder.setContentText(this.textoDelProgreso + " Terminado.");
		if (this.numProcesosPendientes == 0) {
			this.mBuilder.setAutoCancel(true);
			this.reset();
		}
		this.forzarNotificacion();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setIcono(int)
	 */
	@Override
	public void setIcono(int id) {
		this.mBuilder.setSmallIcon(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setId(int)
	 */
	@Override
	public void setId(int ID) {
		this.ID = ID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setInicioProceso()
	 */
	@Override
	public void setInicioProceso() {

		this.mBuilder.setProgress(100, 0, false);
		this.mBuilder.setContentText(this.textoDelProgreso + " Iniciando.");
		this.forzarNotificacion();
	}

	/**
	 * Sets the num procesos.
	 *
	 * @param numProcesos
	 *            el nmero de procesos totales que se van a ejecutar
	 */
	@Override
	public void setNumProcesos(int numProcesos) {
		this.numProcesos = numProcesos;
		this.numProcesosPendientes = numProcesos;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setProgresoProceso(int)
	 */
	@Override
	public void setProgresoProceso(int progresoActual) {
		int progreso = this.calcularProgreso(progresoActual);
		this.mBuilder.setProgress(100, progreso, false);
		this.mBuilder.setContentText(this.textoDelProgreso + " " + progreso + "%");
		this.notifica();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setTexto(java.lang.String)
	 */
	@Override
	public void setTexto(String texto) {
		this.mBuilder.setContentText(texto);
		this.textoDelProgreso = texto;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#setTitulo(java.lang.String)
	 */
	@Override
	public void setTitulo(String title) {
		this.mBuilder.setContentTitle(title);
	}

	/**
	 * Reset.
	 */
	private void reset() {
		this.numProcesos = 1;
		this.numProcesosPendientes = 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.utils.IProgress#getNumProcesosPendientes()
	 */
	@Override
	protected int getNumProcesosPendientes() {
		return this.numProcesosPendientes;
	}
}
